/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * Parse the Ariane command line.
 * Available options are predefined in the static function.
 */
public final class CommandLineParser {
	private CommandLineParser() { }
	
	public static List<String> parse( String[] args, Map<String, Object> options ) {
		final List<String> graphFilenames = new ArrayList<String>();
		int argIndex = 0;
		while (argIndex < args.length) {
			if (args[argIndex].startsWith("-")) {
				if ("-version".equals(args[argIndex]) || "-v".equals(args[argIndex])) {
					System.err.println("Ariane \"" + Ariane.VERSION_ID  + "\"");
					System.exit(0);
				} else if ("-nosplash".equals(args[argIndex])) {
					options.put("nosplash", true);
				} else if ("-config".equals(args[argIndex])) {
					argIndex++;
					if (argIndex == args.length) {
						System.err.println("No configuration directory specified ... option -config ignored.");
					} else {
						options.put("config", args[argIndex]);
					}
				} else if ("-lib".equals(args[argIndex])) {
					argIndex++;
					if (argIndex == args.length) {
						System.err.println("No operator library configuration file specified... option -lib ignored.");
					} else {
						options.put("lib", args[argIndex]);
					}
				} else if ("-help".equalsIgnoreCase(args[argIndex]) || "-h".equalsIgnoreCase(args[argIndex])) {
					System.err.println("Usage: ariane [options] [ariane-file ..]");
					System.err.println("Options include: ");
					System.err.println("  -help -h 		display this message");
					System.err.println("  -version -v	print product version and exit");
					System.err.println("  -nosplash		launch Ariane without the splash screen");
					System.err.println("  -config<dir>	specify the configuration directory path isntead of $HOME/.greyc.");
					System.err.println("  -lib <file>	use<fichier> instead of $PANDOREHOME/lib/pandore.cfg");
					System.exit(0);
				} else {
					System.err.println("Option '" + args[argIndex] + "' unknown option, ignored...");
				}
			} else {
				graphFilenames.add(args[argIndex]);
			}
			argIndex++;
		}
		return graphFilenames;
	}
}
