package fr.greyc.pantheon.ariane.model;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URISyntaxException;
import java.net.URL;

import fr.greyc.pantheon.ariane.Ariane;

/**
 * This class holds the definition of platform specific values.
 */
public final class PlatformSettings {
	private static String _operatingSystem = System.getProperty("os.name").toLowerCase();

	private PlatformSettings() { }

	public static boolean isWindows() {
		return _operatingSystem.indexOf("win") >= 0;
	}

	public static boolean isMac() {
		return _operatingSystem.indexOf("mac") >= 0;
	}

	public static boolean isLinux() {
		return _operatingSystem.indexOf("nux") >= 0;
	}

	public static boolean isUnix() {
		return _operatingSystem.indexOf("nix") >= 0 || _operatingSystem.indexOf("nux") >= 0 || _operatingSystem.indexOf("aix") > 0 || _operatingSystem.indexOf("sunos") >= 0;
	}

	public static boolean isSolaris() {
		return _operatingSystem.indexOf("sunos") >= 0;
	}

	public static String getLibraryPathPropertyName( ) {
		if (PlatformSettings.isMac()) {
			return "DYLD_LIBRARY_PATH";
		} else if (PlatformSettings.isWindows()) {
			return "PATH";
		} else {
			return "LD_LIBRARY_PATH";
		}
	}

	public static String getHomePath() {
		return System.getProperty("user.home", ".");
	}

	public static String getJavaLibraryPath() {
		return System.getProperty("java.library.path");
	}

	public static String getCurrentJarPath() {
		// Tips: getting the absolute file name of the jar.
		Class<?> arianeClass = Ariane.class;
		URL tmp = arianeClass.getResource(arianeClass.getSimpleName() + ".class");
		String filename = tmp.getFile();
		int jarFilenameLength = filename.indexOf('!'); // Jar file uses the ! meta character
		if (jarFilenameLength > 0) {
			filename = filename.substring(0, jarFilenameLength);
		}

		try {
			URL url = new URL(filename);
			return new File(url.toURI()).getParent();
		} catch (URISyntaxException | MalformedURLException e) {
		}
		return new File(filename).getParent();
	}
}