/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.model.graph.item;

import fr.greyc.pantheon.ariane.model.graph.slot.InputParameterSlot;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.presenter.Graph;
import fr.greyc.pantheon.ariane.view.graph.item.ControlItemView;
import fr.greyc.pantheon.ariane.view.graph.item.GraphItemView;

/**
 * This is the base class of all control items.
 * It provides facilities to set the number of parameters, inputs and outputs.
 */
public abstract class ControlItem extends GraphItem {

	public ControlItem( GraphItemView graphItemView, TreeItem treeItem, Graph graph, String name ) {
		super(graphItemView, treeItem, graph, name, null);
		createSlots(0, 0, 0, 0, 1);
	}

	public ControlItem( GraphItemView graphItemView, Graph graph ) {
		super(graphItemView, graph);
		createSlots(0, 0, 0, 0, 1);
	}

	protected final void askProperties( String name, int defaultInputImageNumber, int defaultParameterNumber, int defaultOutputImageNumber ) throws InterruptedException {
		((ControlItemView)getView()).askProperties(name, defaultInputImageNumber, defaultParameterNumber, defaultOutputImageNumber);
	}

	public abstract void setPrototype( String name, int inputImageNumber, int inputParameterNumber, int outputImageNumber );

	@Override
	public final String toString( ) {
		String commandLine = "(" + getId() + "): " + getName();

		for (InputParameterSlot isp : getInputParameterSlots()) {
			if (isp.getValue() != null) {
				commandLine += " " + isp.getValue();
			}
		}
		return commandLine;
	}
}