/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.model.graph.item;

import java.io.IOException;
import java.util.List;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.exception.ArianeMissingInputException;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.graph.slot.InputDataSlot;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.Graph;
import fr.greyc.pantheon.ariane.view.graph.item.ControlStructureItemView;

/**
 * A <code>ControlStrutureMergeItem</code> merges two inputs
 * slots into a unique output slot.
 * The result is chosen as the first validated input.
 */
public final class MergeControlStructureItem extends ControlItem {
	public static final String TAGNAME = "Merge";

	public MergeControlStructureItem( TreeItem treeItem, Graph graph ) {
		super(new ControlStructureItemView(), treeItem, graph, treeItem.getName());
		createSlots(treeItem.getNumberOfInputDataSlots(), treeItem.getNumberOfParameterSlots(), 0, treeItem.getNumberOfOutputDataSlots(), 1);
	}

	public MergeControlStructureItem( Graph graph, List<XMLElement> xmlTree ) throws IOException {
		super(new ControlStructureItemView(), graph);
		createFromXML(xmlTree);
	}

	@Override
	public void setPrototype( String name, int iIm, int iP, int oIm ) {
		/* empty */
	}

	@Override
	public String getTagName() {
		return TAGNAME;
	}

	///// 
	/////  E X E C U T I O N 
	/////

	@Override
	protected boolean primitiveExecute( ) throws ArianeRuntimeException {
		try {
			for (int i = 0; i < getInputDataSlots().size(); i++) {
				InputDataSlot isi = getInputDataSlots().get(i);
				if (isi.hasValue(false)) {
					getOutputDataSlots().get(0).setValue(isi.getValue());
					getOutputResultSlot().setValue(Float.toString(i));
					changeStatus(Status.EXECUTED);
					return true;
				}
			}
		} catch (ArianeMissingInputException e) {
			if (Graph.isExecutionRunning()) {
				Ariane.getPresenter().printErrorMessage(Ariane.ErrorCode.INPUTOUTPUT, getAbsoluteName(), getId(), e.getLocalizedMessage());
				getGraph().stopExecution();
				changeStatus(Status.ERROR);
			}
			throw new ArianeRuntimeException(e);
		}
		return false;
	}
}