/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.model.graph.item.etc;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * This class parses the template string for
 * the argument line of an operator.
 * Argument line uses : $m, $p, $i, $o $p0, $i1 ...[] wildcards.
 */
public final class ArgumentLineParser {
	private static boolean _isString;
	private static boolean _isSet;
	private static List<String> _commandLine;
	private static List<String> _optionalCommandLine;
	private static List<String> _temp;
	private static String _parsedText;
	private static String _format;
	private static int _index;
	private static char _character;

	private ArgumentLineParser() { }

	public static List<String> parse( String command, String customFormat, List<String> maskLine, List<String> parameterLine, List<String> inputLine, List<String> outputLine ) {
		_format = customFormat.replaceAll("\\[", "\\[ ").replaceAll("\\]", " \\]");
		if (_format.trim().isEmpty()) { // for the sake of compatibility: the Pandore default command line
			_format = " [$p] [-m $m] [$i] [$o]";
		}

		_commandLine = new ArrayList<String>();
		_commandLine.add(command);

		_optionalCommandLine = null;
		_temp = _commandLine;
		_parsedText = "";
		_isSet = false;
		_isString = false;

		_index = 0;
		while (_index < _format.length()) {
			_character = _format.charAt(_index++);
			switch (_character) {
			case '"':
				quoteQuote();
				break;

			case ' ':
				space();
				break;

			case '[':
				openBraket();
				break;

			case ']':
				closeBracket();
				break;

			case '$': 
				metaCharacter(maskLine, parameterLine, inputLine, outputLine);
				break;

			default:
				regularCharacter();
				break;
			}
		}
		if (_parsedText.length() > 0) {
			_commandLine.add(_parsedText);
		}

		return removeExtraSpaces(_commandLine);
	}

	private static void regularCharacter() {
		_parsedText += _character;
	}

	private static void space() {
		if (_isString) {
			_parsedText += " ";
		} else {
			while (_index < _format.length()) {
				_character = _format.charAt(_index++);
				if (_character != ' ') {
					_index--;
					break;
				}
			}
			_temp.add(_parsedText);
			_parsedText = "";
		}
	}

	private static void metaCharacter( List<String> maskLine, List<String> parameterLine, List<String> inputLine, List<String> outputLine ) {
		boolean end = false;
		while (!end && _index < _format.length()) {
			_character = _format.charAt(_index++);
			switch (_character) {
			case '$':
				dollar();
				end = true;
				break;

			case '{':
			case '}':
				break;

			case 'm':
				masque(maskLine);
				break;

			case 'p':
				replaceMetaCharacter(parameterLine);
				break;
			case 'i':
				replaceMetaCharacter(inputLine);
				break;
			case 'o':
				replaceMetaCharacter(outputLine);
				break;

			default:
				_index--;
				end = true;
				break;
			}
		}
	}

	private static void replaceMetaCharacter( List<String> parameterLine ) {
		if (_index < _format.length()) {
			_character = _format.charAt(_index++);
			if (Character.isDigit(_character)) {
				int position = _character - '0';
				if (position < parameterLine.size()) {
					_parsedText += parameterLine.get(position);
					_isSet = true;
				}
			} else if (_character == '}') {
				masque(parameterLine);
			} else {
				masque(parameterLine);
				_index--;
			}
		} else {
			masque(parameterLine);
		}
	}

	private static void masque( List<String> maskLine ) {
		_isSet = !maskLine.isEmpty();
		_temp.add(_parsedText);
		_parsedText = "";
		_temp.addAll(maskLine);
	}

	private static void dollar() {
		_parsedText += "$";
	}

	private static void closeBracket() {
		if (_isSet) {
			_temp.add(_parsedText);
			_commandLine.addAll(_temp);
		}
		_parsedText = "";
		_temp = _commandLine;
	}

	private static void openBraket() {
		_optionalCommandLine = new LinkedList<String>();
		_temp.add(_parsedText);
		_parsedText = "";
		_temp = _optionalCommandLine;
		_isSet = false;
	}

	private static void quoteQuote() {
		_isString = !_isString;
		_parsedText += "\"";
	}

	private static List<String> removeExtraSpaces( List<String> commandLine ) {
		List<String> newCommandLine = new LinkedList<String>();
		for (String commandPart : commandLine) { 
			if (!commandPart.trim().isEmpty()) {
				newCommandLine.add(commandPart.trim());
			}
		}
		return newCommandLine;
	}
}
