/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.model.graph.item.etc;

import java.io.File;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.PlatformSettings;

/**
 * <code>OperatorProcessBuilder</code> overrides the ProcessBuilder
 * in order to add some variables in the execution environment.
 * It is an adaptor (in the sense of Design Pattern) of the class ProcessBuilder
 * since ProcessBuilder is a final class.
 */
public final class OperatorProcessBuilder {
	private final ProcessBuilder _processBuilder;

	public OperatorProcessBuilder( List<String> commandLine ) {
		_processBuilder = new ProcessBuilder(commandLine);
		setEnvironment();
	}

	public OperatorProcessBuilder( String command ) {
		_processBuilder = new ProcessBuilder(command);
		setEnvironment();
	}

	/**
	 * Sets the convenient execution environment for both UNIX, Windows and MacOSX
	 * (PATH and LD_LIBRARY_PATH).
	 */
	private void setEnvironment( ) {
		_processBuilder.redirectErrorStream(true);
		Map<String, String> env = _processBuilder.environment();
		env.put("PATH", Ariane.getLibraryPath() + File.separatorChar + "bin" + env.get("PATH"));
		String propertyName = PlatformSettings.getLibraryPathPropertyName();
		env.put(propertyName, env.get(propertyName)
				+ File.pathSeparatorChar + Ariane.getLibraryPath()
				+ File.separatorChar + "lib");
	}

	/**
	 * Starts a new process using the attributes of this
	 * process builder.
	 * 
	 * @return the related process.
	 * @throws IOException in case thread error.
	 */
	public Process start() throws IOException {
		return _processBuilder.start();
	}
}
