/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.model.graph.slot;

import java.io.IOException;

import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.item.DataFile;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.view.graph.slot.OutputDataSlotView;

/**
 * The class <code>OutputDataSlot</code> defines output image slot.
 * It can be connected to several <code>InputDataSlot</code>.
 * An <code>OutputDataSlot</code> can be set with a data file.
 */
public final class OutputDataSlot extends OutputSlot {
	private String _value;

	public OutputDataSlot( GraphItem item, int rank ) {
		super(new OutputDataSlotView(), item, rank);
	}

	@Override
	public String toString() {
		if (getGraphItem() instanceof DataFile) {
			return "OutputDataSlot: " + getGraphItem().getBinary();
		}
		if (_value != null) {
			return "OutputDataSlot: " + _value;
		}
		return "OutputDataSlot: " + getTempName();
	}

	private String createTempFilename() {
		return UserPreferences.createTemporaryName(getGraph().getIdentifier(), getGraphItem().getId(), getRank());
	}

	/**
	 * Creates the complete filename for a temporary file.
	 * <br>Uses the value of temporary directory defined
	 * in the preferences.
	 * <br><i>Does not use the java method <tt>createTempFile</tt>
	 * because it creates the filename in the /temp directory</i>.
	 * 
	 * @return the complete filename.
	 */
	public String getTempName() {
		return UserPreferences.getTemporaryDirectory() + createTempFilename();
	}

	public String getValue() {
		return _value;
	}

	public void setValue() {
		_value = getTempName();
		setValidated();
	}

	public void setValue( String fileName ) {
		_value = fileName;
		setValidated();
	}

	@Override
	public boolean isAcceptableDropSite( Slot slot ) {
		return slot.getGraphItem() != this.getGraphItem()
				&& (slot instanceof InputDataSlot || slot instanceof InputMaskSlot);
	}

	///// 
	///// I N P U T   /   O U T P U T
	///// 

	@Override
	public XMLElement convertToXML( XMLElement element ) throws IOException {
		element.addChildren(new XMLElement("StartItemId", Integer.toString(getGraphItem().getId())));
		element.addChildren(new XMLElement("StartSlotType", "OutSlotImage"));
		element.addChildren(new XMLElement("StartSlotIndex", Integer.toString(getRank())));
		return element;
	}

	public void showDataFileContents() {
		((OutputDataSlotView)getView()).showDataFileContents();
	}
}