/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.model.graph.slot;

import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.view.graph.slot.OutputResultSlotView;

import java.io.IOException;

/**
 * The class <code>OutputResultSlot</code> defines result slot.
 * <br>Result value can be :
 * <ul>
 * <li>SUCCESS
 * <li>FAILURE
 * <li>a float
 * <li>a string.
 * </ul>
 * <br>It can be connected to several <code>InputParameterSlot</code>.
 */
public final class OutputResultSlot extends OutputSlot {
	private String _result;

	public OutputResultSlot( GraphItem item, int rank ) {
		super(new OutputResultSlotView(), item, rank);
	}

	@Override
	public String toString( ) {
		return "OutputResultSlot: " + getValue();
	}

	///// 
	/////  V A L U E 
	/////

	public void setValue( String result ) {
		_result = result;
		setValidated();
	}

	public String getValue() {
		return _result;
	}

	@Override
	public boolean isAcceptableDropSite( Slot slot ) {
		return slot.getGraphItem() != this.getGraphItem() && slot instanceof InputParameterSlot;
	}

	@Override
	public XMLElement convertToXML( XMLElement element ) throws IOException {
		element.addChildren(new XMLElement("StartItemId", Integer.toString(getGraphItem().getId())));
		element.addChildren(new XMLElement("StartSlotType", "OutSlotResult"));
		element.addChildren(new XMLElement("StartSlotIndex", Integer.toString(getRank())));
		return element;
	}
}