/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.model.tree;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.util.List;

import fr.greyc.pantheon.ariane.model.graph.item.ForControlStructureItem;
import fr.greyc.pantheon.ariane.model.graph.item.WhileControlStructureItem;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.model.graph.item.MacroOperatorItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.presenter.Graph;

/**
 * A <code>SubroutineTreeItem</code> is an item that can be inserted in
 * the tree of the explorer.
 */
public final class SubroutineTreeItem extends TreeItem { // TODO
	/** The list of items ion the subgraph. */
	private List<XMLElement> _body;

	/** The magic number for transferable item. */
	public static final DataFlavor[] FLAVORS = { new DataFlavor("application/x-java-serialized-subroutine-item-tree", "SubroutineTreeItem Flavor") };

	public SubroutineTreeItem( List<XMLElement> xmlTree ) {
		super(null, null, null, 0, 0, 0, "" );
		loadItemFromXMLTree(xmlTree);
	}

	@Override
	public DataFlavor[] getTransferDataFlavors( ) {
		return FLAVORS.clone();
	}

	@Override
	public Object getTransferData( DataFlavor flavor ) throws UnsupportedFlavorException, IOException {
		if (isDataFlavorSupported(flavor)) {
			return this;
		} else {
			throw new UnsupportedFlavorException(flavor);
		}
	}

	@Override
	public boolean isDataFlavorSupported( DataFlavor flavor ) {
		return flavor.equals(FLAVORS[0]);
	}

	@Override
	public GraphItem createGraphItem( Graph graph ) {
		try {
			GraphItem item;
			switch (getType()) {
			case MACRO_OPERATOR: item = new MacroOperatorItem(graph, _body, this); break;
			case FOR_CONTROL:    item = new ForControlStructureItem(graph, _body, this); break;
			case WHILE_CONTROL:  item = new WhileControlStructureItem(graph, _body, this); break;
			default: return null;
			}
			item.updateName();
			return item;
		} catch (IOException e) { // NOPMD by regis on 7/20/14 10:14 AM
		}
		return null;
	}

	@Override
	public XMLElement convertToXMLTree( ) throws IOException {
		XMLElement element = new XMLElement("Subroutine");
		element.addChildren(new XMLElement("Category", getCategory()));
		element.addChildren(new XMLElement("Name" , getName()));
		element.addChildren(new XMLElement("Documentation", getDocumentationURL()));
		element.addChildren(new XMLElement("Signature", getSignature()));
		XMLElement item;
		switch (getType()) {
		case MACRO_OPERATOR: item = new XMLElement("MacroOperator"); break;
		case FOR_CONTROL: item = new XMLElement("For"); break;
		case WHILE_CONTROL: item = new XMLElement("While"); break;
		default:
			throw new IOException("Unkown subroutine item type:" + getType());
		}
		item.addChildren(_body);
		element.addChildren(item);
		return element;
	}

	public List<XMLElement> getBody() {
		return _body;
	}

	@Override
	protected void loadItemFromXMLTree( List<XMLElement> xmlelements ) {
		for (XMLElement item : xmlelements ) {
			if ("Category".equalsIgnoreCase(item.getName())) {
				setCategory(item.getValue());
			} else if ("Name".equalsIgnoreCase(item.getName())) {
				setName(item.getValue());
			} else if ("Documentation".equalsIgnoreCase(item.getName())) {
				setDocumentationURL(item.getValue());
			} else if ("Signature".equalsIgnoreCase(item.getName())) {
				setSignature(item.getValue());
			} else if ("MacroOperator".equalsIgnoreCase(item.getName())) {
				setType(Item.MACRO_OPERATOR);
				_body = item.getChildren();
			} else if ("For".equalsIgnoreCase(item.getName())) {
				setType(Item.FOR_CONTROL);
				_body = item.getChildren();
			} else if ("While".equalsIgnoreCase(item.getName())) {
				setType(Item.WHILE_CONTROL);
				_body = item.getChildren();
			}
		}
	}
}