/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.presenter;

import java.io.File;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.graph.GraphModel;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;

/**
 * A <code>RootGraph</code> defines a program.
 */
public final class RootGraph extends Graph { // TODO
	private String _name;
	private boolean _isModified;

	public RootGraph( String name ) {
		_name = name;
	}

	public RootGraph( File file ) {
		super(file);
		_name = getFileBasename(file.getName());
	}

	public RootGraph( String name, GraphModel graphModel ) { // For tests
		super(graphModel);
		_name = name;
	}

	@Override
	public GraphItem getSuperGraphItem( ) {
		return null;
	}

	@Override
	public RootGraph getRoot( ) {
		return this;
	}

	@Override
	public boolean isRoot( ) {
		return true;
	}

	@Override
	public String getName( ) {
		return _name;
	}

	@Override
	protected void setName( String name ) {
		_name = name;
	}

	@Override
	public boolean isModified() {
		return _isModified;
	}

	@Override
	public void setModified( boolean value ) {
		if (isAlive()) {
			Ariane.getPresenter().setModifiedSaveB(value);
			_isModified = value;
		}
	}

	@Override
	public boolean close( ) {
		if (!isModified() || isCloseConfirmed()) {
			closeFrame();
			dispose();
			Ariane.getPresenter().updateRecentFilesListMenu();
			return true;
		} else {
			return false;
		}
	}
}
