/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.presenter;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.model.graph.item.VirtualItem;
import fr.greyc.pantheon.ariane.model.graph.slot.Slot;
import fr.greyc.pantheon.ariane.view.graph.XYConstraint;
import fr.greyc.pantheon.ariane.view.graph.XYLayout;

import java.awt.Point;

/**
 * A <code>SubGraph</code> holds configuration of a complex graph
 * item such as <code>MacroOperator</code> or <code>ControlStructureFor</code>.
 * Its inputs, parameters and outputs are held by a <code>VirtualOperator</code>.
 */
public final class SubGraph extends Graph { // TODO
	private final GraphItem _parent;
	private String _name;
	private final VirtualItem _virtualOperator;

	public SubGraph( GraphItem parent, String name, int inputDataSlotNumber, int inputParameterSlotNumber,
			int outputDataSlotNumber, int intermediateDataSlotNumber, int intermediateParamaterSlotNumber ) {
		_name = name;
		_parent = parent;

		// Builds the related VirtualOperator (identifier=0).
		_virtualOperator = new VirtualItem(this, name, inputDataSlotNumber, inputParameterSlotNumber, outputDataSlotNumber , intermediateDataSlotNumber, intermediateParamaterSlotNumber);
		addItem(_virtualOperator, new Point(0, 0), true);
		for (int i = 0; i <  inputDataSlotNumber; i++) {
			add(_virtualOperator.getVirtualInputDataSlots().get(i), XYLayout.Orientation.WESTNORTH, i);
		}
		for (int i = 0; i <  inputParameterSlotNumber; i++) {
			add(_virtualOperator.getVirtualInputParameterSlots().get(i), XYLayout.Orientation.NORTHWEST, i);
		}
		add(_virtualOperator.getVirtualInputMaskSlot(), XYLayout.Orientation.NORTHWEST, inputParameterSlotNumber);
		for (int i = 0; i <  outputDataSlotNumber; i++) {
			add(_virtualOperator.getVirtualOutputDataSlots().get(i), XYLayout.Orientation.NORTHEAST, i);
		}
		add(_virtualOperator.getVirtualOutputResultSlot(), XYLayout.Orientation.SOUTHEAST, 0);
		for (int i = 0; i <  intermediateDataSlotNumber; i++) {
			add(_virtualOperator.getVirtualIntermediateDataSlots(i), XYLayout.Orientation.EASTSOUTH, i);
		}
		for (int i = 0; i <  intermediateParamaterSlotNumber; i++) {
			add(_virtualOperator.getVirtualIntermediateResultSlot(i), XYLayout.Orientation.SOUTHWEST, i);
		}
	}

	@Override
	public GraphItem getSuperGraphItem() {
		return _parent;
	}

	@Override
	public Graph getRoot( ) {
		return _parent.getGraph().getRoot();
	}

	@Override
	public boolean isRoot( ) {
		return false;
	}

	@Override
	public String getName( ) {
		if (_parent == null) {
			return null;
		}
		return _parent.getGraph().getName() + "::" + _name;
	}

	@Override
	protected void setName( String name ) {
		_name = name;
	}

	public VirtualItem getVirtualItem( ) {
		return _virtualOperator;
	}

	@Override
	public boolean isModified( ) {
		return getRoot().isModified();
	}

	@Override
	public void setModified( boolean value ) {
		getRoot().setModified(value);
	}

	public void add( Slot slot, Object alignment, int index ) {
		XYConstraint constraint = new XYConstraint((XYLayout.Orientation)alignment, index);
		getView().add(slot.getView(), constraint);
	}

	public void showContent( ) {
		Ariane.getPresenter().addGraph(this);
	}

	@Override
	public boolean close( ) {
		closeFrame();
		return true;
	}
}