/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.presenter;

import javax.swing.undo.CompoundEdit;
import javax.swing.undo.UndoManager;
import javax.swing.undo.UndoableEdit;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;

/**
 * UndoRedoManager manages a list of UndoableEdits, providing a way
 * to undo or redo the appropriate actions.
 */
public final class UndoRedoManager extends UndoManager {
	private CompoundEdit _edit;

	public UndoRedoManager() {
		setLimit(UserPreferences.getHistorySize());
	}

	public void validateAction( UndoableEdit action ) {
		if (_edit == null) {
			_edit = new CompoundEdit();
		}
		_edit.addEdit(action);
	}

	public void startActionEdition( ) {
		_edit = new CompoundEdit();
	}

	public void endActionEdition( Graph graph ) {
		if (_edit != null && _edit.isSignificant()) {
			_edit.end();
			addEdit(_edit);
		}
		graph.setModified(true);
		Ariane.getPresenter().update();
	}

	public void cancelCurrentActionEdition() {
		_edit = null;
	}

	@Override
	public void undo( ) {
		try {
			super.undo();
			Ariane.getPresenter().update();
		} catch (javax.swing.undo.CannotUndoException e) {
			Ariane.getPresenter().printErrorMessage(Ariane.getString("Graph.history.error1"));
		}
	}

	@Override
	public void redo( ) {
		try {
			super.redo();
			Ariane.getPresenter().update();
		} catch (javax.swing.undo.CannotRedoException e) {
			Ariane.getPresenter().printErrorMessage(Ariane.getString("Graph.history.error1"));
		}
	}
}
