/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.presenter.command;

import java.awt.Point;

import javax.swing.undo.AbstractUndoableEdit;

import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.presenter.Graph;

/**
 * This class is part of the history manager.
 * It stores a move item action.
 */
public final class MoveItemEdit extends AbstractUndoableEdit {
	/** The related graph. */
	private final Graph _graph;
	/** The related item. */
	private final GraphItem _item;
	/** The source coordinates. */
	private final Point _to;
	/** The destination coordinates. */
	private final Point _from;

	/**
	 * Creates a new edit.
	 * 
	 * @param graph a graph.
	 * @param item a graph item.
	 * @param from the start location.
	 * @param to the destination location.
	 */
	public MoveItemEdit( Graph graph, GraphItem item, Point from, Point to ) {
		_graph = graph;
		_item = item;
		_to = to;
		_from = from;
	}

	/**
	 * Re-applies the edit, assuming that it has been undone.
	 */
	@Override
	public void redo( ) {
		_graph.moveItem(_item, _to, false);
		_graph.revalidate();
	}

	/**
	 * Undoes the edit that was made.
	 */
	@Override
	public void undo( ) {
		_graph.moveItem(_item, _from, false);
		_graph.revalidate();
	}
}