/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.Ariane;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Font;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;

import com.sun.management.UnixOperatingSystemMXBean;

import java.lang.management.ManagementFactory;
import java.lang.management.OperatingSystemMXBean;
import java.text.DecimalFormat;

/**
 * A <tt>AboutBox</tt> displays general information about
 * Ariane such as authors, version number, memory statistics, etc.
 */
public final class AboutBox extends JDialog {
	public AboutBox( final JFrame parent ) {
		super(parent, Ariane.getString("AboutBox.title"), true);
		final JTabbedPane tabbedPane = new JTabbedPane();

		// Tab 1: Logo
		{
			final JPanel panel = new JPanel();
			final Box panel1 = Box.createVerticalBox();
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.version.number")));
			panel1.add(new JLabel(Ariane.VERSION_ID, JLabel.CENTER));
			panel1.add(Box.createVerticalStrut(10));
			panel1.add(new JLabel(Ariane.getIcon("splash.png")), BorderLayout.CENTER);
			panel1.add(Box.createVerticalStrut(10));
			panel1.add(Box.createGlue());
			panel.add(panel1);
			tabbedPane.addTab(Ariane.getString("AboutBox.version.title"), panel);
		}
		getContentPane().add(tabbedPane);
		pack();
		setResizable(false);

		// Tab 2: version
		{
			final Box panel = Box.createVerticalBox();

			final JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel.add(panel1);
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.warning")));
			final String text = Ariane.getString("AboutBox.license.text");
			final JTextArea textArea = new JTextArea(text);
			final Font font = new Font("Serif", Font.PLAIN, 13);
			textArea.setFont(font);
			textArea.setEditable(false);
			panel1.add(textArea);
			tabbedPane.addTab(Ariane.getString("AboutBox.license.title"), panel);
		}

		// Tab 3 authors
		{
			final Box panel = Box.createVerticalBox();
			final JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.contribution")));
			final Box box1 = Box.createVerticalBox();
			for (int i = 0; i < Ariane.AUTHORS.length; i++) {
				box1.add(new JLabel(" - " + Ariane.AUTHORS[i]));
			}
			panel1.add(box1);
			panel.add(panel1);

			final JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.addressTitle")));
			final Box box2 = Box.createVerticalBox();
			box2.add(new JLabel(Ariane.getString("AboutBox.team")));
			box2.add(new JLabel(Ariane.getString("AboutBox.lab")));
			box2.add(new JLabel(Ariane.getString("AboutBox.address")));
			box2.add(new JLabel(Ariane.getString("AboutBox.zipcode")));
			box2.add(new JLabel(Ariane.getString("AboutBox.http")));
			panel2.add(box2);
			panel.add(panel2);
			tabbedPane.addTab(Ariane.getString("AboutBox.authors.title"), panel);
		}

		// Tab 4 statistics
		{
			final Box panel = Box.createVerticalBox();
			System.gc(); // NOPMD by regis on 7/20/14 11:33 AM
			JPanel statics;
			final DecimalFormat format = new DecimalFormat("#.##");
			double stat;

			statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
			statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.statistics.usedMemory")));
			stat = Runtime.getRuntime().totalMemory() / (1024.0 * 1024.0);
			statics.add(new JLabel(Ariane.getString("AboutBox.statistics.size") + " " + format.format(stat) + Ariane.getString("AboutBox.statistics.megabytes")));
			panel.add(statics);

			statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
			statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.statistics.usedPercent")));
			final double percent = ((double)Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / Runtime.getRuntime().maxMemory();
			statics.add(new JLabel(Ariane.getString("AboutBox.statistics.size") + " " + format.format(percent) + "%"));
			panel.add(statics);

			statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
			statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.statistics.availableProcessors")));
			statics.add(new JLabel(Ariane.getString("AboutBox.statistics.number") + " " + Runtime.getRuntime().availableProcessors()));
			panel.add(statics);

			final OperatingSystemMXBean operatingSystemMXBean = ManagementFactory.getOperatingSystemMXBean();

			if (operatingSystemMXBean instanceof UnixOperatingSystemMXBean) {
				final UnixOperatingSystemMXBean operatingSystem = (UnixOperatingSystemMXBean)operatingSystemMXBean;
				statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
				statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.statistics.maxFileDescriptorCount")));
				statics.add(new JLabel(Ariane.getString("AboutBox.statistics.number") + " " + operatingSystem.getMaxFileDescriptorCount()));
				panel.add(statics);

				statics = new JPanel(new FlowLayout(FlowLayout.LEFT));
				statics.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.statistics.openFileDescriptorCount")));
				statics.add(new JLabel(Ariane.getString("AboutBox.statistics.number") + " " + operatingSystem.getOpenFileDescriptorCount()));
				panel.add(statics);
			}

			tabbedPane.addTab(Ariane.getString("AboutBox.statistics.title"), panel);
		}

		// Tab 5: Legend.
		{
			final Box panel = Box.createVerticalBox();

			final JPanel panel1 = new JPanel(new BorderLayout());
			panel.add(panel1);
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("AboutBox.legend.header")));
			final String text = Ariane.getString("AboutBox.legend.text");
			final JEditorPane textArea = new JEditorPane("text/html", text);
			textArea.setEditable(false);
			panel1.add(textArea, BorderLayout.CENTER);
			tabbedPane.addTab(Ariane.getString("AboutBox.legend.title"), panel);
		}
		setLocationRelativeTo(parent);
	}
}
