package fr.greyc.pantheon.ariane.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JFrame;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.WindowConstants;
import javax.swing.UIManager.LookAndFeelInfo;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.presenter.Presenter;

/**
 * This frame is the main frame that supports all the software.
 */
public final class ArianeFrame extends JFrame {
	private Dimension _size;

	public void initFrame( final Presenter presenter ) {
		final Workspace workspace = presenter.getWorkspace();
		setContentPane(workspace);
		setTitle(Ariane.FRAME_TITLE);
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing( WindowEvent event ) {
				if (workspace.isQuitConfirmed()) {
					((JFrame)(event.getWindow())).setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
					System.exit(0);
				} else {
					((JFrame)(event.getWindow())).setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
				}
			}
		});
	}

	public Dimension getFinalSize() {
		if (getExtendedState() ==  Frame.MAXIMIZED_BOTH) {
			return _size;
		} else {
			return super.getSize();
		}
	}

	public void restoreFrameSize( ) {
		_size = UserPreferences.getFrameSize();
		if (_size != null) {
			setPreferredSize(_size);
			setSize(_size);
		}
		if (UserPreferences.isFrameMaximized()) {
			setExtendedState(Frame.MAXIMIZED_BOTH);
		}
		setLocationByPlatform(true);
	}

	public void installUI() {
		try {
			UIManager.put("control", new Color(215, 214, 223));
			UIManager.put("nimbusBlueGrey", new Color(167, 167, 190));
			UIManager.put("nimbusBase", new Color(42, 42, 140));
			for (LookAndFeelInfo info : UIManager.getInstalledLookAndFeels()) {
				if ("Nimbus".equals(info.getName())) {
					UIManager.setLookAndFeel(info.getClassName());
					break;
				}
			}
		} catch (ReflectiveOperationException | UnsupportedLookAndFeelException e) {
		}

		UIManager.put("OptionPane.yesIcon", Ariane.getIcon("gtk-yes.png"));
		UIManager.put("OptionPane.noIcon", Ariane.getIcon("gtk-no.png"));
		UIManager.put("OptionPane.cancelIcon", Ariane.getIcon("gtk-cancel.png"));
		UIManager.put("OptionPane.okIcon", Ariane.getIcon("gtk-apply.png"));
	}
}
