/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.Ariane;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.JTextPane;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SpinnerNumberModel;
import javax.swing.WindowConstants;
import javax.swing.border.EtchedBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.DefaultEditorKit;
import javax.swing.text.Document;
import javax.swing.text.MutableAttributeSet;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledEditorKit;
import javax.swing.text.html.HTML;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTMLEditorKit.InsertHTMLTextAction;
import javax.swing.undo.CannotRedoException;
import javax.swing.undo.CannotUndoException;
import javax.swing.undo.UndoManager;

/**
 * <code>HTMLDocumentEditor.java</code> is a dialog window
 * to edit HTML documents.
 */
public final class HTMLDocumentEditor extends JDialog {
	public static final int OK_OPTION = JOptionPane.OK_OPTION;
	private static final Dimension DEFAULT_TAB_SIZE = new Dimension(500, 300);

	private HTMLDocument _document;
	private JTextPane _htmlEditorPane;
	private JTextArea _textEditorPane;
	private JTabbedPane _tabbedPane;
	private File _currentFile;
	private int _answer;

	private final UndoableEditListener _undoHandler = new UndoHandler();
	private final UndoManager _undoManager = new UndoManager();
	private UndoAction _undoAction;
	private RedoAction _redoAction;

	public HTMLDocumentEditor( JDialog parent ) {
		super(parent, null, true);
		HTMLEditorKit editorKit = new HTMLEditorKit();
		_document = (HTMLDocument)editorKit.createDefaultDocument();
		initDialog();
	}

	public int edit() {
		startExistingDocument();
		_htmlEditorPane.requestFocusInWindow();
		setVisible(true);
		return _answer;
	}

	public File getSelectedFile() {
		return _currentFile;
	}

	public void setSelectedFile( File file ) {
		_currentFile = file;
	}

	private void initDialog() {
		JToolBar toolbar1 = createMenuToolbar();
		JToolBar toolbar2 = createEditToolbar();

		_document.addUndoableEditListener(_undoHandler);
		resetUndoManager();

		_tabbedPane = new JTabbedPane();

		JScrollPane htmlEditorScrollPane = createHTMLTab();
		_tabbedPane.addTab("HTML", null, htmlEditorScrollPane, "HTML View");

		JScrollPane textEditorScrollPane = createTEXTTab();
		_tabbedPane.addTab("TEXT", null, textEditorScrollPane, "Text View");

		_tabbedPane.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged( ChangeEvent event ) {
				updateEditView();
			}
		});

		JPanel toolbarPanel = new JPanel();
		toolbarPanel.setLayout(new BorderLayout());
		toolbarPanel.add(toolbar1, BorderLayout.NORTH);
		toolbarPanel.add(toolbar2, BorderLayout.CENTER);

		getContentPane().add(toolbarPanel, BorderLayout.NORTH);
		getContentPane().add(_tabbedPane, BorderLayout.CENTER);

		pack();
		setLocationRelativeTo(getParent());
		addWindowListener(new FrameListener());
	}

	private JScrollPane createTEXTTab() {
		_textEditorPane = new JTextArea();
		JScrollPane textEditorScrollPane = new JScrollPane(_textEditorPane);
		textEditorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		textEditorScrollPane.setPreferredSize(DEFAULT_TAB_SIZE);
		return textEditorScrollPane;
	}

	private JScrollPane createHTMLTab() {
		_htmlEditorPane = new JTextPane(_document);
		_htmlEditorPane.setContentType("text/html; charset=ISO-8859-1");
		JScrollPane htmlEditorScrollPane = new JScrollPane(_htmlEditorPane);
		htmlEditorScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		htmlEditorScrollPane.setPreferredSize(DEFAULT_TAB_SIZE);
		return htmlEditorScrollPane;
	}

	private JToolBar createMenuToolbar() {
		JToolBar toolbar1 = new JToolBar();
		toolbar1.setFloatable(false);
		toolbar1.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
		toolbar1.setBorderPainted(true);
		toolbar1.setOpaque(false);

		EditorAction quitA = new EditorAction("Quit", Ariane.getIcon("logout.png")) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				quitConfirmed();
			}
		};
		toolbar1.add(new JToolbarButton(quitA));

		toolbar1.addSeparator();

		EditorAction cutAction = new EditorAction("Cut", Ariane.getIcon("html/cut.png"), new DefaultEditorKit.CutAction());
		EditorAction copyAction = new EditorAction("Copy", Ariane.getIcon("html/copy.png"), new DefaultEditorKit.CopyAction());
		EditorAction pasteAction = new EditorAction("Paste", Ariane.getIcon("html/paste.png"), new DefaultEditorKit.PasteAction());
		toolbar1.add(new JToolbarButton(cutAction));
		toolbar1.add(new JToolbarButton(copyAction));
		toolbar1.add(new JToolbarButton(pasteAction));

		toolbar1.addSeparator();

		_undoAction = new UndoAction(Ariane.getIcon("html/undo.png"));
		_redoAction = new RedoAction(Ariane.getIcon("html/redo.png"));
		toolbar1.add(new JToolbarButton(_undoAction));
		toolbar1.add(new JToolbarButton(_redoAction));
		return toolbar1;
	}

	private JToolBar createEditToolbar() {
		JToolBar toolbar2 = new JToolBar();
		toolbar2.setFloatable(false);
		toolbar2.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
		toolbar2.setBorderPainted(true);
		toolbar2.setOpaque(false);

		ListElement[] blockElements = ListElement.createBlockElementList();
		JComboBox<ListElement> blockElementMenu = new JComboBox<ListElement>(blockElements);
		blockElementMenu.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				@SuppressWarnings("unchecked")
				JComboBox<ListElement> comboBox = (JComboBox<ListElement>)event.getSource();
				ListElement element = (ListElement)comboBox.getSelectedItem();
				element.actionPerformed(event);
			}
		});
		toolbar2.add(blockElementMenu);

		toolbar2.addSeparator();

		EditorAction boldAction = new EditorAction("Bold", Ariane.getIcon("html/bold.png"), new StyledEditorKit.BoldAction());
		EditorAction italicAction = new EditorAction("Italic", Ariane.getIcon("html/italic.png"), new StyledEditorKit.ItalicAction());
		EditorAction underlineAction = new EditorAction("Underline", Ariane.getIcon("html/underline.png"), new StyledEditorKit.UnderlineAction());
		EditorAction strikeThroughAction = new EditorAction("StrikeThrough", Ariane.getIcon("html/strikethrough.png"), new StrikeThroughAction());
		EditorAction subscriptAction = new EditorAction("Subscript", Ariane.getIcon("html/subscript.png"), new SubscriptAction());
		EditorAction superscriptAction = new EditorAction("Superscript", Ariane.getIcon("html/superscript.png"), new SuperscriptAction());
		toolbar2.add(new JToolbarToggleButton(boldAction));
		toolbar2.add(new JToolbarToggleButton(italicAction));
		toolbar2.add(new JToolbarToggleButton(underlineAction));
		toolbar2.add(new JToolbarToggleButton(strikeThroughAction));
		toolbar2.add(new JToolbarToggleButton(subscriptAction));
		toolbar2.add(new JToolbarToggleButton(superscriptAction));

		toolbar2.addSeparator();

		EditorAction alignLeft = new EditorAction("Alignleft", Ariane.getIcon("html/align_left.png"), new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_LEFT));
		EditorAction alignCenter = new EditorAction("AlignCenter", Ariane.getIcon("html/align_center.png"), new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_CENTER));
		EditorAction alignRight = new EditorAction("AlignRight", Ariane.getIcon("html/align_right.png"), new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_RIGHT));
		EditorAction alignJustified = new EditorAction("AlignJustifed", Ariane.getIcon("html/align_justified.png"), new StyledEditorKit.AlignmentAction(null, StyleConstants.ALIGN_JUSTIFIED));
		toolbar2.add(new JToolbarToggleButton(alignLeft));
		toolbar2.add(new JToolbarToggleButton(alignCenter));
		toolbar2.add(new JToolbarToggleButton(alignRight));
		toolbar2.add(new JToolbarToggleButton(alignJustified ));

		String[] fontTypes = {"Default", "SansSerif", "Serif", "Monospaced", "Dialog", "DialogInput"};
		JComboBox<String> fontTypeMenu = new JComboBox<String>(fontTypes);
		fontTypeMenu.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				@SuppressWarnings("unchecked")
				JComboBox<String> cb = (JComboBox<String>)event.getSource();
				String font = (String)cb.getSelectedItem();
				Action action = new StyledEditorKit.FontFamilyAction(font, font);
				action.actionPerformed(event);
			}
		});

		toolbar2.add(fontTypeMenu);
		Integer[] fontSizes = {6, 8, 10, 12, 14, 16, 20, 24, 32, 36, 48, 72};
		JComboBox<Integer> fontSizeMenu = new JComboBox<Integer>(fontSizes);
		fontSizeMenu.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				@SuppressWarnings("unchecked")
				JComboBox<Integer> cb = (JComboBox<Integer>)event.getSource();
				Integer size = (Integer)cb.getSelectedItem();
				Action action = new StyledEditorKit.FontSizeAction(String.valueOf(size), size);
				action.actionPerformed(event);
			}
		});
		toolbar2.add(fontSizeMenu);
		fontSizeMenu.setSelectedIndex(4);
		Pair[] colors = { new Pair("Black", Color.BLACK),
				new Pair("Red", Color.RED),
				new Pair("Orange", Color.ORANGE),
				new Pair("Yellow", Color.YELLOW), 
				new Pair("Green", Color.GREEN),
				new Pair("Blue", Color.BLUE),
				new Pair("Cyan", Color.CYAN),
				new Pair("Magenta", Color.MAGENTA)};
		JComboBox<Pair> colorBox = new JComboBox<Pair>(colors);
		colorBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				@SuppressWarnings("unchecked")
				JComboBox<Pair> cb = (JComboBox<Pair>)event.getSource();
				Pair pair = (Pair)cb.getSelectedItem();
				Action action = new StyledEditorKit.ForegroundAction(pair._name, pair._color);
				action.actionPerformed(event);
			}
		});
		toolbar2.add(colorBox);

		EditorAction breakRule = new EditorAction("InsertBreak", Ariane.getIcon("html/br.png"), new HTMLEditorKit.InsertHTMLTextAction("InsertBreak", "<br>", HTML.Tag.BODY, HTML.Tag.BR));
		toolbar2.add(new JToolbarButton(breakRule));
		EditorAction hRule = new EditorAction("InsertHR", Ariane.getIcon("html/hrule.png"), new HTMLEditorKit.InsertHTMLTextAction("InsertHR", "<hr>", HTML.Tag.BODY, HTML.Tag.HR));
		toolbar2.add(new JToolbarButton(hRule));
		EditorAction unorderedList = new EditorAction("UnorderedList", Ariane.getIcon("html/listunordered.png"), new InsertHTMLTextAction("Itemize", "<ul><li></li></ul>", HTML.Tag.UL, HTML.Tag.LI, HTML.Tag.BODY, HTML.Tag.UL));
		toolbar2.add(new JToolbarButton(unorderedList));
		EditorAction orderedList = new EditorAction("OrderedList", Ariane.getIcon("html/listordered.png"), new InsertHTMLTextAction("Enumerate", "<ol><li></li></ol>", HTML.Tag.UL, HTML.Tag.LI, HTML.Tag.BODY, HTML.Tag.OL));
		toolbar2.add(new JToolbarButton(orderedList));
		toolbar2.add(new JToolbarButton(new LinkAction(Ariane.getIcon("html/link.png"))));
		toolbar2.add(new JToolbarButton(new ImageAction(Ariane.getIcon("html/image.png"))));
		toolbar2.add(new JToolbarButton(new TableAction(Ariane.getIcon("html/table.png"))));

		toolbar2.addSeparator();
		return toolbar2;
	}

	private void updateEditView() {
		if (_tabbedPane.getSelectedIndex() == 0) {
			_htmlEditorPane.setText("");
			insertHTML(_htmlEditorPane, _textEditorPane.getText(), 0);
		} else {
			_textEditorPane.setText(_htmlEditorPane.getText());
		}
	}

	private void insertHTML( JEditorPane editor, String html, int location ) {
		try {
			HTMLEditorKit kit = (HTMLEditorKit)editor.getEditorKit();
			Document doc = editor.getDocument();
			StringReader reader = new StringReader(html);
			kit.read(reader, doc, location);
		} catch (IOException | BadLocationException ex) {
		}
	}

	private void resetUndoManager() {
		_undoManager.discardAllEdits();
		_undoAction.update();
		_redoAction.update();
	}

	protected void startExistingDocument( ) {
		HTMLEditorKit editorKit = new HTMLEditorKit();
		_document = (HTMLDocument)editorKit.createDefaultDocument();
		_document.putProperty("IgnoreCharsetDirective", Boolean.TRUE);

		try {
			if (_currentFile == null) {
				throw new IOException();
			}
			FileReader fileReader = new FileReader(_currentFile);
			editorKit.read(fileReader, _document, 0);
			fileReader.close();
			setTitle(_currentFile.getName());
		} catch (IOException | BadLocationException e) {
			_currentFile = null;
			setTitle(Ariane.getString("HTMLViewer.title"));
		}
		Document oldDoc = _htmlEditorPane.getDocument();
		oldDoc.removeUndoableEditListener(_undoHandler);
		_document.addUndoableEditListener(_undoHandler);
		_htmlEditorPane.setDocument(_document);
		resetUndoManager();
	}

	private boolean quitConfirmed() {
		_answer = JOptionPane.showConfirmDialog(this, Ariane.getString("HTMLDocumentEditor.quitMessage"));
		if (_answer == JOptionPane.CANCEL_OPTION) {
			return false;
		} else if (_answer == JOptionPane.OK_OPTION) {
			try {
				saveDocument();
			} catch (IOException e) {
				JOptionPane.showMessageDialog(this, Ariane.getString("HTMLViewer.saveErrorMessage"), Ariane.getString("HTMLViewer.saveErrorTitle"), JOptionPane.ERROR_MESSAGE);
				return false;
			}
		}
		setVisible(false);
		dispose();
		return true;
	}

	private void saveDocument() throws IOException {
		if (_currentFile == null) {
			saveDocumentAs();
		} else {
			saveFile();
		}
	}

	private void saveFile() throws IOException {
		FileWriter fw = new FileWriter(_currentFile);
		fw.write(_htmlEditorPane.getText());
		fw.close();
	}

	private void saveDocumentAs() throws IOException {
		File current = new File(".");
		JFileChooser chooser = new JFileChooser(current);
		chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		chooser.setFileFilter(new HTMLFileFilter());
		int approval = chooser.showSaveDialog(this);
		if (approval == JFileChooser.APPROVE_OPTION) {
			File newFile = chooser.getSelectedFile();
			if (newFile.exists()) {
				String message = newFile.getAbsolutePath() + Ariane.getString("HTMLDocumentEditor.overrideMessage");
				if (JOptionPane.showConfirmDialog(this, message) == JOptionPane.YES_OPTION) {
					_currentFile = newFile;
					setTitle(_currentFile.getName());
					saveFile();
				}
			} else {
				_currentFile = new File(newFile.getAbsolutePath());
				setTitle(_currentFile.getName());
				saveFile();
			}
		}
	}

	class FrameListener extends WindowAdapter {
		@Override
		public void windowClosing( WindowEvent event ) {
			if (quitConfirmed()) {
				((JDialog)(event.getWindow())).setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
			} else {
				((JDialog)(event.getWindow())).setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
			}
		}
	}

	class SubscriptAction extends StyledEditorKit.StyledTextAction {
		public SubscriptAction() {
			super(StyleConstants.Subscript.toString());
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			JEditorPane editor = getEditor(event);
			if (editor != null) {
				StyledEditorKit kit = getStyledEditorKit(editor);
				MutableAttributeSet attr = kit.getInputAttributes();
				boolean subscript = StyleConstants.isSubscript(attr) ? false : true;
				SimpleAttributeSet sas = new SimpleAttributeSet();
				StyleConstants.setSubscript(sas, subscript);
				setCharacterAttributes(editor, sas, false);
			}
		}
	}

	class SuperscriptAction extends StyledEditorKit.StyledTextAction {
		public SuperscriptAction() {
			super(StyleConstants.Superscript.toString());
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			JEditorPane editor = getEditor(event);
			if (editor != null) {
				StyledEditorKit kit = getStyledEditorKit(editor);
				MutableAttributeSet attr = kit.getInputAttributes();
				boolean superscript = StyleConstants.isSuperscript(attr) ? false : true;
				SimpleAttributeSet sas = new SimpleAttributeSet();
				StyleConstants.setSuperscript(sas, superscript);
				setCharacterAttributes(editor, sas, false);
			}
		}
	}

	class StrikeThroughAction extends StyledEditorKit.StyledTextAction {
		public StrikeThroughAction() {
			super(StyleConstants.StrikeThrough.toString());
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			JEditorPane editor = getEditor(event);
			if (editor != null) {
				StyledEditorKit kit = getStyledEditorKit(editor);
				MutableAttributeSet attr = kit.getInputAttributes();
				boolean strikeThrough = StyleConstants.isStrikeThrough(attr) ? false : true;
				SimpleAttributeSet sas = new SimpleAttributeSet();
				StyleConstants.setStrikeThrough(sas, strikeThrough);
				setCharacterAttributes(editor, sas, false);
			}
		}
	}

	class HTMLFileFilter extends javax.swing.filechooser.FileFilter {
		@Override
		public boolean accept( File file ) {
			return file.isDirectory() || file.getName().toLowerCase().indexOf(".htm") > 0;
		}

		@Override
		public String getDescription() {
			return "html";
		}
	}

	final class UndoHandler implements UndoableEditListener {
		@Override
		public void undoableEditHappened( UndoableEditEvent event ) {
			_undoManager.addEdit(event.getEdit());
			_undoAction.update();
			_redoAction.update();
		}
	}

	class UndoAction extends AbstractAction {
		public UndoAction( Icon icon ) {
			super("Undo", icon);
			setEnabled(false);
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			try {
				_undoManager.undo();
			} catch (CannotUndoException ex) {
			}
			update();
			_redoAction.update();
		}

		protected void update() {
			if (_undoManager.canUndo()) {
				setEnabled(true);
				putValue(Action.NAME, _undoManager.getUndoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, "Undo");
			}
		}
	}

	class RedoAction extends AbstractAction {
		public RedoAction( Icon icon ) {
			super("Redo", icon);
			setEnabled(false);
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			try {
				_undoManager.redo();
			} catch (CannotRedoException e) {
				//				System.err.println("Unable to redo: " + ex);
				//				ex.printStackTrace();
			}
			update();
			_undoAction.update();
		}

		protected void update() {
			if (_undoManager.canRedo()) {
				setEnabled(true);
				putValue(Action.NAME, _undoManager.getRedoPresentationName());
			} else {
				setEnabled(false);
				putValue(Action.NAME, "Redo");
			}
		}
	}

	static final class ListElement extends HTMLEditorKit.InsertHTMLTextAction {
		private ListElement( String name, String html, HTML.Tag parentTag, HTML.Tag addTag ) {
			super(name, html, parentTag, addTag, null, null);
		}

		private static ListElement[] createBlockElementList() {
			ListElement[] listElements = {
					new ListElement("Body Text", "<div> </div>", HTML.Tag.BODY, HTML.Tag.DIV),
					new ListElement("Paragraph", "<p> </p>", HTML.Tag.BODY, HTML.Tag.P),
					new ListElement("Preformatted", "<pre> </pre>", HTML.Tag.BODY, HTML.Tag.PRE),
					new ListElement("Heading 1", "<h1> </h1>", HTML.Tag.BODY, HTML.Tag.H1),
					new ListElement("Heading 2", "<h2> </h2>", HTML.Tag.BODY, HTML.Tag.H2),
					new ListElement("Heading 3", "<h3> </h3>", HTML.Tag.BODY, HTML.Tag.H3),
					new ListElement("Heading 4", "<h4> </h4>", HTML.Tag.BODY, HTML.Tag.H4),
					new ListElement("Heading 5", "<h5> </h5>", HTML.Tag.BODY, HTML.Tag.H5),
					new ListElement("Heading 6", "<h6> </h6>", HTML.Tag.BODY, HTML.Tag.H6) };
			return listElements;
		}

		@Override
		public String toString() {
			return (String)getValue(Action.NAME);
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			super.actionPerformed(event);
			JEditorPane pane = getEditor(event);
			pane.requestFocusInWindow();
		}
	}

	private class EditorAction extends AbstractAction {
		private final Action _action;

		public EditorAction( String name, Icon icon ) {
			super(name, icon);
			_action = null;
		}

		public EditorAction( String name, Icon icon, Action action ) {
			super(name, icon);
			_action = action;
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			_action.actionPerformed(event);
			_htmlEditorPane.requestFocusInWindow();
		}
	}

	private final class TableAction extends EditorAction {
		public TableAction( Icon icon ) {
			super(Ariane.getString("HTMLViewer.table"), icon);
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			JPanel panel = new JPanel(new GridLayout(2, 2));
			JSpinner rowEditor = new JSpinner(new SpinnerNumberModel(1, 1, 10, 1));
			JSpinner colEditor = new JSpinner(new SpinnerNumberModel(1, 1, 10, 1));
			panel.add(new JLabel(Ariane.getString("HTMLViewer.column")));
			panel.add(colEditor);
			panel.add(new JLabel(Ariane.getString("HTMLViewer.row")));
			panel.add(rowEditor);

			if (JOptionPane.showConfirmDialog(HTMLDocumentEditor.this, panel, "Table", JOptionPane.OK_CANCEL_OPTION) == JOptionPane.OK_OPTION) {
				HTMLEditorKit kit = (HTMLEditorKit)_htmlEditorPane.getEditorKit();
				HTMLDocument doc = (HTMLDocument)_htmlEditorPane.getDocument();
				int numRows = (Integer)rowEditor.getValue();
				int numCols = (Integer)colEditor.getValue();

				String html = "<table border=1>";
				for (int row = 0; row < numRows; row++) {
					html += "<tr>\n"; 
					for (int col = 0; col < numCols; col++) {
						html += "\t<td>\n</td>\n";
					}
					html += "</tr>\n";
				}
				html += "</table>";
				try {
					kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), html, 0, 0, HTML.Tag.TABLE);
				} catch (IOException | BadLocationException e) {
				}
			}
		}
	}

	private final class LinkAction extends EditorAction {
		public LinkAction( Icon icon ) {
			super(Ariane.getString("HTMLViewer.link"), icon);
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			HTMLEditorKit kit = (HTMLEditorKit)_htmlEditorPane.getEditorKit();
			HTMLDocument doc = (HTMLDocument)_htmlEditorPane.getDocument();
			String value = JOptionPane.showInputDialog(HTMLDocumentEditor.this, "URL:");
			try {
				kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), "<a href=\"" + value + "\">", 0, 0, HTML.Tag.A);
			} catch (IOException | BadLocationException e) {
			}
			_htmlEditorPane.requestFocusInWindow();
		}
	}

	private final class ImageAction extends EditorAction {
		public ImageAction( Icon icon ) {
			super(Ariane.getString("HTMLViewer.image"), icon);
		}

		@Override
		public void actionPerformed( ActionEvent event ) {
			HTMLEditorKit kit = (HTMLEditorKit)_htmlEditorPane.getEditorKit();
			HTMLDocument doc = (HTMLDocument)_htmlEditorPane.getDocument();
			String value = JOptionPane.showInputDialog(HTMLDocumentEditor.this, "Image file:");
			try {
				kit.insertHTML(doc, _htmlEditorPane.getCaretPosition(), "<img src=\"" + value + "\">", 0, 0, HTML.Tag.IMG);
			} catch (IOException | BadLocationException e) {
			}
		}
	}

	private final class JToolbarToggleButton extends JToggleButton {
		public JToolbarToggleButton( Action action ) {
			super(action);
			setText(null);
			setToolTipText((String)action.getValue(Action.NAME));
		}
	}

	private final class JToolbarButton extends JButton {
		public JToolbarButton( Action action ) {
			super(action);
			setText(null);
			setToolTipText((String)action.getValue(Action.NAME));
		}
	}

	final class Pair {
		private final String _name;
		private final Color _color;

		public Pair( String name, Color color ) {
			_name = name;
			_color = color;
		}

		@Override
		public String toString() {
			return _name;
		}
	}

//	public static void main( String[] args ) {
//		Ariane.setCurrentLocale(new Locale("fr", "FR"));
//		HTMLDocumentEditor editor = new HTMLDocumentEditor(null);
//		editor.setSelectedFile(new File("new File(/home/regis/pantheon/software/pandore/doc/operators/fr/operatorsP0/padd.html"));
//		editor.edit();
//	}
}