/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.List;
import java.util.Map;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Box;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JSeparator;
import javax.swing.KeyStroke;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.presenter.Presenter;

public final class MenuBar extends JMenuBar {
	private final JMenu _fileMenu;
	private final JMenu _recentFilesSubmenu;
	private final JMenu _editMenu;
	private final JMenu _optionsMenu;
	private final JMenu _graphMenu;
	private final JMenu _helpMenu;

	private final JMenuItem _newI;
	private final JMenuItem _openI;
	private final JMenuItem _saveAsI;
	private final JMenuItem _saveI;
	private final JMenuItem _printI;
	private final JMenuItem _importSubroutineI;
	private final JMenuItem _exportSubroutineI;
	private final JMenuItem _quitI;

	private final JMenuItem _closeI;
	private final JMenuItem _closeAllI;

	private final JMenuItem _undoI;
	private final JMenuItem _redoI;
	private final JMenuItem _copyI;
	private final JMenuItem _cutI;
	private final JMenuItem _pasteI;
	private final JMenuItem _eraseI;
	private final JMenuItem _selectI;

	private final JMenuItem _confI;
	private final JMenuItem _prefI;
	private final JMenuItem _operatorTreeI;
	private final JMenuItem _subroutineTreeI;

	private final JMenuItem _execI;
	private final JMenuItem _resetI;
	private final JMenuItem _generateI;
	private final JMenuItem _stopI;

	private final JMenuItem _aboutI;
	private final JMenuItem _helpI;

	public MenuBar( Map<String, Action> actions, JMenu explorerSubMenu, List<String> recentlyClosedWorkspaceFilenames, Presenter presenter ) { 
		_fileMenu = new JMenu();
		_fileMenu.setMnemonic('F');
		
		_newI = _fileMenu.add(createMenuItem(actions.get("new"), KeyEvent.VK_N));
		_openI = _fileMenu.add(createMenuItem(actions.get("open"), KeyEvent.VK_O));
		_recentFilesSubmenu = new JMenu();
		_recentFilesSubmenu.setMnemonic('I');
		updateRecentlyClosedWorkspacesListMenu(recentlyClosedWorkspaceFilenames, presenter);
		if (recentlyClosedWorkspaceFilenames.isEmpty()) {
			_recentFilesSubmenu.setEnabled(false);
		}
		_fileMenu.add(_recentFilesSubmenu);
		_fileMenu.add(new JSeparator());
		_closeI = _fileMenu.add(createMenuItem(actions.get("close"), KeyEvent.VK_W));
		_closeAllI = _fileMenu.add(new JMenuItem(actions.get("closeAll")));
		_fileMenu.add(new JSeparator());
		_saveI = _fileMenu.add(createMenuItem(actions.get("save"), KeyEvent.VK_S));
		_saveAsI = _fileMenu.add(new JMenuItem(actions.get("saveAs")));
		_fileMenu.add(new JSeparator());
		_importSubroutineI = new JMenuItem(actions.get("importSubroutine"));
		_fileMenu.add(_importSubroutineI);
		_exportSubroutineI = new JMenuItem(actions.get("exportSubroutine"));
		_fileMenu.add(_exportSubroutineI);
		_fileMenu.add(new JSeparator());
		_printI = _fileMenu.add(createMenuItem(actions.get("print"), KeyEvent.VK_P));
		_fileMenu.add(_printI);
		_fileMenu.add(new JSeparator());
		_quitI = _fileMenu.add(createMenuItem(actions.get("quit"), KeyEvent.VK_Q));
		add(_fileMenu);

		// Edit
		_editMenu = new JMenu();
		_editMenu.setMnemonic('E');
		_undoI = _editMenu.add(createMenuItem(actions.get("undo"), KeyEvent.VK_Z));
		_redoI = _editMenu.add(createMenuItem(actions.get("redo"), KeyEvent.VK_Y));
		_editMenu.add(new JSeparator());
		_copyI = _editMenu.add(createMenuItem(actions.get("copy"), KeyEvent.VK_C));
		_cutI = _editMenu.add(createMenuItem(actions.get("cut"), KeyEvent.VK_X));
		_pasteI = _editMenu.add(createMenuItem(actions.get("paste"), KeyEvent.VK_V));
		_eraseI = _editMenu.add(createMenuItem(actions.get("erase"), KeyEvent.VK_DELETE, 0));
		_editMenu.add(new JSeparator());
		_selectI = _editMenu.add(createMenuItem(actions.get("select"), KeyEvent.VK_A));
		add(_editMenu);

		// Options
		_optionsMenu = new JMenu();
		_optionsMenu.setMnemonic('O');

		_confI = _optionsMenu.add(new JMenuItem(actions.get("conf")));
		_prefI = _optionsMenu.add(new JMenuItem(actions.get("preferences")));
		_operatorTreeI = _optionsMenu.add(new JMenuItem(actions.get("operatorTree")));
		_subroutineTreeI = _optionsMenu.add(new JMenuItem(actions.get("subroutineTree")));
		add(_optionsMenu);

		add(explorerSubMenu);

		// Graph
		_graphMenu = new JMenu();
		_graphMenu.setMnemonic('G');
		
		_execI = _graphMenu.add(createMenuItem(actions.get("exec"), KeyEvent.VK_R));
		_resetI = _graphMenu.add(createMenuItem(actions.get("reset"), KeyEvent.VK_U));
		_stopI = _graphMenu.add(createMenuItem(actions.get("stop"), KeyEvent.VK_G));
		_generateI = _graphMenu.add(new JMenuItem(actions.get("generate")));

		add(_graphMenu);

		add(Box.createHorizontalGlue());

		// Help
		_helpMenu = new JMenu();
		_helpMenu.setMnemonic('H');

		_helpI = _helpMenu.add(createMenuItem(actions.get("help"), KeyEvent.VK_F1, 0));
		_aboutI = _helpMenu.add(new JMenuItem(actions.get("about")));

		add(_helpMenu);
	}

	private JMenuItem createMenuItem(Action newA , int vkN ) {
		return createMenuItem(newA, vkN, KeyEvent.CTRL_MASK);
	}


	private JMenuItem createMenuItem(Action newA , int vkN, int toto ) {
		JMenuItem menuItem = new JMenuItem(newA);
		menuItem.setAccelerator(KeyStroke.getKeyStroke(vkN, toto));
		return menuItem;
	}

	protected void updateRecentlyClosedWorkspacesListMenu( List<String> recentlyClosedWorkspaceFilenames, final Presenter presenter ) {
		_recentFilesSubmenu.removeAll();
		int fileIndex = 1;
		for (String filename : recentlyClosedWorkspaceFilenames ) {
			final String file = filename;
			Action action = new AbstractAction(fileIndex + " " + filename) {
				@Override
				public void actionPerformed( ActionEvent event ) {
					presenter.openFile(file);
				}
			};
			JMenuItem item = new JMenuItem(action);
			item.setMnemonic((char)('0' + fileIndex));
			_recentFilesSubmenu.add(item);
			fileIndex++;
		}
		_recentFilesSubmenu.setEnabled(true);
	}

	protected void setInternationalText( ) {
		//Menu file
		_fileMenu.setText(Ariane.getString("Menu.file.title"));
		_recentFilesSubmenu.setText(Ariane.getString("Menu.file.recentFiles"));
		_newI.setText(Ariane.getString("Menu.file.new"));
		_openI.setText(Ariane.getString("Menu.file.open"));
		_saveAsI.setText(Ariane.getString("Menu.file.saveas"));
		_saveI.setText(Ariane.getString("Menu.file.save"));
		_importSubroutineI.setText(Ariane.getString("Menu.file.importSubroutine"));
		_exportSubroutineI.setText(Ariane.getString("Menu.file.exportSubroutine"));
		_printI.setText(Ariane.getString("Menu.file.print"));
		_closeI.setText(Ariane.getString("Menu.file.close"));
		_closeAllI.setText(Ariane.getString("Menu.file.closeAll"));
		_quitI.setText(Ariane.getString("Menu.file.quit"));

		//Menu edit 
		_editMenu.setText(Ariane.getString("Menu.edit.title"));
		_undoI.setText(Ariane.getString("Menu.edit.undo"));
		_redoI.setText(Ariane.getString("Menu.edit.redo"));
		_copyI.setText(Ariane.getString("Menu.edit.copy"));
		_cutI.setText(Ariane.getString("Menu.edit.cut"));
		_pasteI.setText(Ariane.getString("Menu.edit.paste"));
		_eraseI.setText(Ariane.getString("Menu.edit.delete"));
		_selectI.setText(Ariane.getString("Menu.edit.selectall"));

		//Menu preferences
		_optionsMenu.setText(Ariane.getString("Menu.options.title"));
		_confI.setText(Ariane.getString("Menu.options.configuration"));
		_prefI.setText(Ariane.getString("Menu.options.preferences"));
		_operatorTreeI.setText(Ariane.getString("Menu.options.operators"));
		_subroutineTreeI.setText(Ariane.getString("Menu.options.subroutines"));

		//Menu graph
		_graphMenu.setText(Ariane.getString("Menu.graph.title"));
		_execI.setText(Ariane.getString("Menu.graph.exec"));
		_resetI.setText(Ariane.getString("Menu.graph.reset"));
		_generateI.setText(Ariane.getString("Menu.graph.generate"));
		_stopI.setText(Ariane.getString("Menu.graph.stop"));

		// Menu help
		_helpMenu.setText(Ariane.getString("Menu.help.title"));
		_aboutI.setText(Ariane.getString("Menu.help.about"));
		_helpI.setText(Ariane.getString("Menu.help.help"));
	}
}

