/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view;

import fr.greyc.pantheon.ariane.Ariane;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;
import javax.swing.border.EtchedBorder;

/**
 * A <code>SplashScreen</code> displays a given image during the initialization
 * of the software.
 */
public final class SplashScreen extends JWindow {
	private final ProgressBar _progressBar = new ProgressBar();

	public SplashScreen( final String imageName ) {
		final ClassLoader classLoader = getClass().getClassLoader();
		final ImageIcon splashImage = new ImageIcon(classLoader.getResource(imageName));

		final JLabel splashButton = new JLabel("");
		if (splashImage != null) {
			final int imgWidth = splashImage.getIconWidth();
			final int imgHeight = splashImage.getIconHeight();
			final Dimension scrSize = Toolkit.getDefaultToolkit().getScreenSize();
			setLocation(scrSize.width / 2 - imgWidth / 2, scrSize.height / 2 - imgHeight / 2);
			splashButton.setIcon(addReleaseNumberOntoImage(splashImage));
		}
		setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		getContentPane().setLayout(new BorderLayout(0, 0));
		getContentPane().add(splashButton, BorderLayout.CENTER);
		getContentPane().add(_progressBar, BorderLayout.SOUTH);
		setAlwaysOnTop(true);
		pack();
	}

	private ImageIcon addReleaseNumberOntoImage( final ImageIcon imageIcon ) {
		final Image image = imageIcon.getImage();
		final int width = image.getWidth(null);
		final int height = image.getHeight(null);
		final BufferedImage bimg = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		bimg.createGraphics().drawImage(image, 0, 0, this);
		bimg.getGraphics().drawString("Version: " + Ariane.VERSION_ID, 80, 220);

		return new ImageIcon(bimg);
	}

	public void showProgress( final String message, final int percent ) {
		_progressBar.showStatus(message);
		_progressBar.showProgress(percent);
	}

	private final class ProgressBar extends JPanel {
		private final JLabel _messageText = new JLabel();
		private final JProgressBar _progressInidicator = new JProgressBar();

		public ProgressBar( ) {
			_progressInidicator.setMinimum(0);
			_progressInidicator.setMaximum(100);
			_progressInidicator.setMinimumSize(new Dimension(100, 20));
			_progressInidicator.setSize(new Dimension(100, 20));

			_messageText.setMinimumSize(new Dimension(300, 20));
			_messageText.setSize(new Dimension(300, 20));
			_messageText.setFont(_messageText.getFont().deriveFont(Font.BOLD, 9f));
			_messageText.setForeground(Color.black);

			setLayout(new BorderLayout());
			setBorder(new EtchedBorder(EtchedBorder.LOWERED));
			add(_messageText, BorderLayout.CENTER);
			add(_progressInidicator, BorderLayout.EAST);
		}

		public void showStatus( final String message ) {
			_messageText.setText(message);
			paintImmediately(getBounds());
		}

		public void showProgress( final int percent ) {
			_progressInidicator.setValue(percent);
		}
	}
}