/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view;

import java.awt.Color;
import java.awt.FlowLayout;
import java.util.Map;

import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.border.EtchedBorder;
import javax.swing.border.LineBorder;

import fr.greyc.pantheon.ariane.Ariane;

public final class Toolbar extends JPanel {
	private final JButton _newB;
	private final JButton _openB;
	private final JButton _saveB;
	private final JButton _saveAsB;
	private final JButton _printB;

	private final JButton _undoB;
	private final JButton _redoB;
	private final JButton _cutB;
	private final JButton _copyB;
	private final JButton _pasteB;
	private final JButton _eraseB;

	private final JButton _preferencesB;
	private final JButton _operatorTreeB;
	private final JButton _subroutineTreeB;

	private final JButton _executeB;
	private final JButton _generateB;
	private final JButton _stopB;

	private final JButton _helpB;

	public Toolbar( Map<String, Action> actions ) {
		super(new FlowLayout(FlowLayout.LEFT));
		setBorder(new LineBorder(Color.BLACK));
		JToolBar toolbar = buildToolBar();
		_newB = (JButton)toolbar.add(createButton(actions.get("new")));
		_openB = (JButton)toolbar.add(createButton(actions.get("open")));
		_saveB = (JButton)toolbar.add(createButton(actions.get("save")));
		_saveAsB = (JButton)toolbar.add(createButton(actions.get("saveAs")));
		toolbar.addSeparator();
		_printB = (JButton)toolbar.add(createButton(actions.get("print")));
		toolbar.addSeparator();
		_preferencesB = (JButton)toolbar.add(createButton(actions.get("preferences")));
		_operatorTreeB = (JButton)toolbar.add(createButton(actions.get("operatorTree")));
		_subroutineTreeB = (JButton)toolbar.add(createButton(actions.get("subroutineTree")));
		toolbar.addSeparator();
		_undoB = (JButton)toolbar.add(createButton(actions.get("undo")));
		_redoB = (JButton)toolbar.add(createButton(actions.get("redo")));
		_copyB = (JButton)toolbar.add(createButton(actions.get("copy")));
		_cutB = (JButton)toolbar.add(createButton(actions.get("cut")));
		_pasteB = (JButton)toolbar.add(createButton(actions.get("paste")));
		_eraseB = (JButton)toolbar.add(createButton(actions.get("erase")));
		toolbar.addSeparator();
		_executeB = (JButton)toolbar.add(createButton(actions.get("exec")));
		_stopB = (JButton)toolbar.add(createButton(actions.get("stop")));
		toolbar.addSeparator();
		_generateB = (JButton)toolbar.add(createButton(actions.get("generate")));
		toolbar.addSeparator();
		_helpB = (JButton)toolbar.add(createButton(actions.get("help")));

		add(toolbar);
	}

	private JToolBar buildToolBar( ) {
		JToolBar toolbar = new JToolBar();
		toolbar.setFloatable(false);
		toolbar.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
		toolbar.setBorderPainted(true);
		toolbar.setOpaque(true);
		toolbar.setBackground(toolbar.getBackground().darker());

		return toolbar;
	}

	private JButton createButton( Action action ) {
		JButton button  = new JButton(action);
		button.setBorderPainted(false);
		return button;
	}

	protected void setInternationalText( ) {
		_newB.setToolTipText(Ariane.getString("Menu.file.new"));
		_openB.setToolTipText(Ariane.getString("Menu.file.open"));
		_saveB.setToolTipText(Ariane.getString("Menu.file.save"));
		_saveAsB.setToolTipText(Ariane.getString("Menu.file.saveas"));
		_printB.setToolTipText(Ariane.getString("Menu.file.print"));
		_undoB.setToolTipText(Ariane.getString("Menu.edit.undo"));
		_redoB.setToolTipText(Ariane.getString("Menu.edit.redo"));
		_copyB.setToolTipText(Ariane.getString("Menu.edit.copy"));
		_cutB.setToolTipText(Ariane.getString("Menu.edit.cut"));
		_pasteB.setToolTipText(Ariane.getString("Menu.edit.paste"));
		_eraseB.setToolTipText(Ariane.getString("Menu.edit.delete"));
		_preferencesB.setToolTipText(Ariane.getString("Menu.options.preferences"));
		_operatorTreeB.setToolTipText(Ariane.getString("Menu.options.operators"));
		_subroutineTreeB.setToolTipText(Ariane.getString("Menu.options.subroutines"));
		_executeB.setToolTipText(Ariane.getString("Menu.graph.exec"));
		_generateB.setToolTipText(Ariane.getString("Menu.graph.generate"));
		_helpB.setToolTipText(Ariane.getString("Menu.help.help"));
		_stopB.setToolTipText(Ariane.getString("Menu.graph.stop"));
	}
}
