/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view; // NOPMD by regis on 4/24/15 6:23 PM

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.view.graph.GraphView;
import fr.greyc.pantheon.ariane.view.graph.item.etc.ControlColor;
import fr.greyc.pantheon.ariane.view.graph.item.etc.ControlStructureColor;
import fr.greyc.pantheon.ariane.view.graph.item.etc.DataFileColor;
import fr.greyc.pantheon.ariane.view.graph.item.etc.NoteColor;
import fr.greyc.pantheon.ariane.view.graph.item.etc.OperatorColor;
import fr.greyc.pantheon.ariane.view.utils.GDialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingUtilities;
import javax.swing.border.EtchedBorder;

/**
 * The class <code>UserPreferencesChooser</code> allows the user
 * to set the preferences.
 */
public final class UserPreferencesChooser extends GDialog {
	private static final String ARIANE_BROWSE = "Ariane.browse";
	private static final int NUMBER_OF_COLORS = 9;

	private boolean _isValidated;

	private String _language;
	private int _defaultFontSize;
	private String _visuCommand;
	private String _statusCommand;

	private String _temporaryFileExtension = ".tmp";
	private Color[] _colorArray;
	private String[] _colorNameArray;
	private String _temporaryDirectory;
	private String _fileDirectory;
	private String _binaryDirectory;
	private int _historySize;

	private final JTabbedPane _prefsTabbedPane = new JTabbedPane();
	private static int _lastestSelectedTab;

	private final JTextField _browserPath = new JTextField(10);
	private final JTextField _editorPath = new JTextField(10);
	private final JCheckBox _toolTipState = new JCheckBox();

	private JSpinner _undoSizeSpinner;

	public UserPreferencesChooser( ) {
		super(Ariane.getFrame(), Ariane.getString("PreferenceChooser"), true);
		setResizable(false);
		displayContent();
		pack();
		setLocationRelativeTo(getOwner());
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing( WindowEvent event ) {
				cancelAction();
			}
		});
	}

	private Box createGeneralPreferencesTab() {
		Box panel = Box.createVerticalBox();
		JPanel panel1 = new JPanel();
		panel.add(panel1);
		panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
		panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.general.language")));

		// List of known languages.
		panel1.add(new JLabel(Ariane.getString("PreferenceChooser.general.language.label")));
		String[] languages = {"Français", "English"};
		JComboBox<String> list = new JComboBox<String>(languages);
		list.setSelectedItem(_language);
		list.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				@SuppressWarnings("unchecked")
				JComboBox<String> comboBox = (JComboBox<String>)event.getSource();
				_language = (String)comboBox.getSelectedItem();
			}
		});
		panel1.add(list);

		JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(panel2);
		panel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.general.tooltip")));
		panel2.add(_toolTipState);
		panel2.add(new JLabel(Ariane.getString("PreferenceChooser.general.tooltip.text")));

		JPanel panel3 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(panel3);
		panel3.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(EtchedBorder.RAISED), Ariane.getString("PreferenceChooser.undoHistoryTitle")));
		SpinnerModel spinnerModel;
		spinnerModel = new SpinnerNumberModel(_historySize, 10, 500, 1);
		_undoSizeSpinner = new JSpinner(spinnerModel);
		panel3.add(new JLabel(Ariane.getString("PreferenceChooser.undoHistoryText")));
		panel3.add(_undoSizeSpinner);

		// Browser
		JPanel panel4 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(panel4);
		panel4.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.general.browser")));
		panel4.add(new JLabel(Ariane.getString("PreferenceChooser.general.browser.text")));
		panel4.add(_browserPath);
		JButton browser1 = new JButton(Ariane.getString(ARIANE_BROWSE));
		panel4.add(browser1);
		browser1.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				selectSoftware(_browserPath);
			}
		});

		// Editor
		JPanel panel5 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(panel5);
		panel5.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.general.editor")));
		panel5.add(new JLabel(Ariane.getString("PreferenceChooser.general.editor.text")));
		panel5.add(_editorPath);
		JButton browser2 = new JButton(Ariane.getString(ARIANE_BROWSE));
		panel5.add(browser2);
		browser2.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				selectSoftware(_editorPath);
			}
		});

		panel.add(Box.createGlue());
		return panel;
	}

	private void selectSoftware( JTextField textfield ) {
		final JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
		fileChooser.setDialogTitle(Ariane.getString("PreferenceChooser.general.selectProgram"));
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.setCurrentDirectory(new File("."));
		if (fileChooser.showOpenDialog(UserPreferencesChooser.this) == JFileChooser.APPROVE_OPTION) {
			textfield.setText(fileChooser.getSelectedFile().getAbsolutePath());
		}
	}

	private String selectDirectory( String directory, final JTextField textfield ) {
		final JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
		fileChooser.setDialogTitle(Ariane.getString("FileChooser.titleSelectDir"));
		fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
		if (directory != null) {
			fileChooser.setCurrentDirectory(new File(directory));
		}
		if (fileChooser.showOpenDialog(UserPreferencesChooser.this) == JFileChooser.APPROVE_OPTION) {
			String nDirectory = fileChooser.getSelectedFile().getAbsolutePath();
			textfield.setText(nDirectory);
			return nDirectory;
		} else {
			return directory;
		}
	}

	private Box createRuntimePreferencesTab() {
		Box panel = Box.createVerticalBox();

		// -> VISU command
		JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(panel1);
		panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.runtime.visuCommand")));
		final JTextField textfield2 = new JTextField(20);
		panel1.add(new JLabel(Ariane.getString("PreferenceChooser.runtime.CommandName")));

		textfield2.setText(_visuCommand);
		textfield2.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost( FocusEvent event ) {
				_visuCommand = ((JTextField)event.getSource()).getText();
			}
		});
		panel1.add(textfield2);

		// STATUS command
		JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(panel2);
		panel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.runtime.statusCommand")));
		panel2.add(new JLabel(Ariane.getString("PreferenceChooser.runtime.CommandName")));
		final JTextField textfield3 = new JTextField(20);
		textfield3.setText(_statusCommand);
		textfield3.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost( FocusEvent event ) {
				_statusCommand = ((JTextField)event.getSource()).getText();
			}
		});
		panel2.add(textfield3);
		return panel;
	}

	private Box createGraphPreferencesTab() {
		Box panel = Box.createVerticalBox();

		// List of known languages.
		JPanel panel1 = new JPanel();
		panel.add(panel1);
		panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
		panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.graph.font")));

		panel1.add(new JLabel(Ariane.getString("PreferenceChooser.graph.font.size")));
		String[] fontSize = {"8", "9", "10", "11", "12"};
		JComboBox<String> list = new JComboBox<String>(fontSize);
		list.setSelectedItem(Integer.toString(_defaultFontSize));
		list.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				@SuppressWarnings("unchecked")
				JComboBox<String> comboBox = (JComboBox<String>)event.getSource();
				_defaultFontSize = Integer.parseInt((String)comboBox.getSelectedItem());
			}
		});
		panel1.add(list);
		panel1.add(new JLabel(Ariane.getString("PreferenceChooser.graph.font.notice")));
		panel.add(panel1);

		createColorSquare(panel, 5, NUMBER_OF_COLORS);

		return panel;
	}

	private Box createColorPreferencesTab() {
		Box panel = Box.createVerticalBox();
		createColorSquare(panel, 0, 5);
		return panel;
	}

	private void createColorSquare( Box panel, int start, int end ) {
		for (int i = start; i < end; i++) {
			JPanel panel1 = new JPanel(new FlowLayout(FlowLayout.LEFT));
			panel.add(panel1);
			panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), _colorNameArray[i]));
			panel1.add(new BindingColor(Ariane.getString("PreferenceChooser.backgroundColor"), i));
		}
	}

	private Box createPathPreferencesTab() {
		Box panel = Box.createVerticalBox();
		JPanel panel1 = temporaryPath();
		panel.add(panel1);

		JPanel panel2 = temporaryFileExtension(panel);
		panel.add(panel2);
		panel.add(Box.createGlue());

		JPanel panel3 = fileDirectory();
		panel.add(panel3);

		JPanel panel4 = personalOperatorsPath();
		panel.add(panel4);

		return panel;
	}

	private JPanel personalOperatorsPath() {
		JPanel panel4 = new JPanel();
		panel4.setLayout(new FlowLayout(FlowLayout.LEFT));
		panel4.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.path.binaryDirectory")));
		JButton button = new JButton(Ariane.getString(ARIANE_BROWSE));
		final JTextField textfield4 = new JTextField(20);
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				_binaryDirectory = selectDirectory(_binaryDirectory, textfield4);
			}
		});
		textfield4.setText(_binaryDirectory);
		textfield4.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost( FocusEvent event ) {
				_binaryDirectory = ((JTextField)event.getSource()).getText();
				if (_binaryDirectory.trim().isEmpty()) {
					_binaryDirectory = null;
				}
			}
		});
		panel4.add(textfield4);
		panel4.add(button);
		return panel4;
	}

	private JPanel fileDirectory() {
		JPanel panel3 = new JPanel();
		panel3.setLayout(new FlowLayout(FlowLayout.LEFT));
		panel3.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.path.fileDirectory")));
		JButton button3 = new JButton(Ariane.getString(ARIANE_BROWSE));
		final JTextField textfield3 = new JTextField(20);
		button3.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				_fileDirectory = selectDirectory(_fileDirectory, textfield3);
			}
		});
		textfield3.setText(_fileDirectory);
		textfield3.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost( FocusEvent event ) {
				_fileDirectory = ((JTextField)event.getSource()).getText();
				if (_fileDirectory.trim().isEmpty()) {
					_fileDirectory = null;
				}
			}
		});
		panel3.add(textfield3);
		panel3.add(button3);
		return panel3;
	}

	private JPanel temporaryFileExtension( Box panel ) {
		JPanel panel2 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel2.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.runtime.TemporaryFileExtension")));
		JTextField textfield2 = new JTextField(20);
		panel2.add(new JLabel(Ariane.getString("PreferenceChooser.runtime.fileExtension")));

		textfield2.setText(_temporaryFileExtension);
		textfield2.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost( FocusEvent event ) {
				_temporaryFileExtension = ((JTextField)event.getSource()).getText();
			}
		});
		panel2.add(textfield2);
		panel.add(Box.createGlue());

		JPanel panel21 = new JPanel(new FlowLayout(FlowLayout.LEFT));
		panel.add(panel21);
		panel21.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.environment.title")));
		// -> Remove temporary files
		panel21.add(new JLabel(Ariane.getString("PreferenceChooser.environment.cleaning")));
		JButton button2 = new JButton(Ariane.getString("PreferenceChooser.environment.erasing"));
		button2.addActionListener(new ActionListener() { // Erases all temporary files.
			@Override
			public void actionPerformed( ActionEvent event ) {
				int numberOfFiles = UserPreferences.deleteTemporaryFiles();
				if (numberOfFiles > 0) {
					JOptionPane.showMessageDialog(UserPreferencesChooser.this,
							numberOfFiles + Ariane.getString("PreferenceChooser.message1"),
							Ariane.getString("notice"),
							JOptionPane.INFORMATION_MESSAGE);
				}
			}
		} );
		panel21.add(button2);
		return panel2;
	}

	private JPanel temporaryPath() {
		JPanel panel1 = new JPanel();
		panel1.setLayout(new FlowLayout(FlowLayout.LEFT));
		panel1.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("PreferenceChooser.path.temporaryDirectory")));
		JButton button1 = new JButton(Ariane.getString(ARIANE_BROWSE));
		final JTextField textfield1 = new JTextField(20);
		button1.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				_temporaryDirectory = selectDirectory(_temporaryDirectory, textfield1);
			}
		});
		textfield1.setText(_temporaryDirectory);
		textfield1.addFocusListener(new FocusAdapter() {
			@Override
			public void focusLost( FocusEvent event ) {
				_temporaryDirectory = ((JTextField)event.getSource()).getText();
			}
		});
		panel1.add(textfield1);
		panel1.add(button1);
		return panel1;
	}

	private void displayContent() {
		initializePreferences();

		_prefsTabbedPane.addTab(Ariane.getString("PreferenceChooser.general.title"), createGeneralPreferencesTab());
		_prefsTabbedPane.addTab(Ariane.getString("PreferenceChooser.runtime.title"), createRuntimePreferencesTab());
		_prefsTabbedPane.addTab(Ariane.getString("PreferenceChooser.graph.title"), createGraphPreferencesTab());
		_prefsTabbedPane.addTab(Ariane.getString("PreferenceChooser.item.title"), createColorPreferencesTab());
		_prefsTabbedPane.addTab(Ariane.getString("PreferenceChooser.paths"), createPathPreferencesTab());
		_prefsTabbedPane.setSelectedIndex(_lastestSelectedTab);

		// Buttons panel
		final JPanel buttonPanel = new JPanel();
		JButton applyButton = new JButton(Ariane.getString("PreferenceChooser.applyText"), Ariane.getIcon("gtk-apply.png"));
		applyButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				buttonPanel.requestFocusInWindow();
				// Tip: handle event before this event.
				SwingUtilities.invokeLater(new Runnable() {
					@Override
					public void run() {
						applyAction();
					}
				});
			}
		});

		// cancel button
		JButton cancelButton = new JButton();
		cancelButton.setAction(new AbstractAction(Ariane.getString("TreeEditor.cancel"), Ariane.getIcon("gtk-cancel.png")) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				cancelAction();
			}
		});

		JButton helpButton = new JButton();
		helpButton.setAction(new AbstractAction(null, Ariane.getIcon("help.png")) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				Ariane.getPresenter().displayDocumentationPage("Ariane", "manual.html", "#tab" + _prefsTabbedPane.getSelectedIndex());
			}
		});
		buttonPanel.add(helpButton);
		buttonPanel.add(Box.createHorizontalStrut(80));
		buttonPanel.add(applyButton);
		buttonPanel.add(cancelButton);
		setDefaultButton(applyButton);
		setCancelButton(cancelButton);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		getContentPane().add(_prefsTabbedPane, BorderLayout.CENTER);
		pack();
	}

	private void initializePreferences() {
		_language = UserPreferences.getLanguage();
		_toolTipState.setSelected(UserPreferences.isToolTipEnabled());
		_historySize = UserPreferences.getHistorySize();
		_browserPath.setText(UserPreferences.getWebBrowserPath());
		_editorPath.setText(UserPreferences.getTextEditorPath());

		_visuCommand = UserPreferences.getCommandVisu();
		_statusCommand = UserPreferences.getCommandStatus();

		_colorArray = new Color[NUMBER_OF_COLORS];
		_colorNameArray = new String[NUMBER_OF_COLORS];
		int index = 0;
		_colorArray[index] = NoteColor.getColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.note");

		_colorArray[index] = DataFileColor.getColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.datafile");

		_colorArray[index] = OperatorColor.getColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.operator");

		_colorArray[index] = ControlColor.getColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.controlItem");

		_colorArray[index] = ControlStructureColor.getColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.controlStructureItem");

		_colorArray[index] = GraphView.getBackgroundColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.background");

		_colorArray[index] = GraphView.getImageLinkColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.imagelinks");

		_colorArray[index] = GraphView.getValueLinkColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.numericlinks");

		_colorArray[index] = GraphView.getMaskLinkColor();
		_colorNameArray[index++] = Ariane.getString("PreferenceChooser.masklinks");

		_defaultFontSize = UserPreferences.getFontSize();

		_temporaryDirectory = UserPreferences.getTemporaryDirectoryBase();
		_fileDirectory = UserPreferences.getPreferredFileDirectory();
		_temporaryFileExtension = UserPreferences.getTemporaryFileExtension();
		_binaryDirectory = UserPreferences.getPreferredBinaryDirectory();
	}

	private void cancelAction() {
		setVisible(false);
		dispose();
	}

	private void applyAction() {
		changePreferences();
		_lastestSelectedTab = _prefsTabbedPane.getSelectedIndex();
		_isValidated = true;
		setVisible(false);
		dispose();
	}

	public boolean isValidated() {
		return _isValidated;
	}

	private void changePreferences() {
		UserPreferences.changeLanguage(_language);
		UserPreferences.setHistorySize((Integer)_undoSizeSpinner.getValue());
		UserPreferences.setToolTipState(_toolTipState.isSelected());
		UserPreferences.setWebBrowserPath(_browserPath.getText().trim());
		UserPreferences.setTextEditorPath(_editorPath.getText().trim());

		UserPreferences.setCommandVisu(_visuCommand);
		UserPreferences.setCommandStatus(_statusCommand);

		int index = 0;
		NoteColor.setColor(_colorArray[index++]);
		DataFileColor.setColor(_colorArray[index++]);
		OperatorColor.setColor(_colorArray[index++]);
		ControlColor.setColor(_colorArray[index++]);
		ControlStructureColor.setColor(_colorArray[index++]);

		GraphView.setBackgroundColor(_colorArray[index++]);
		GraphView.setImageLinkColor(_colorArray[index++]);
		GraphView.setValueLinkColor(_colorArray[index++]);
		GraphView.setMaskLinkColor(_colorArray[index++]);

		UserPreferences.setFontSize(_defaultFontSize);
		try {
			UserPreferences.setTemporaryDirectoryBase(_temporaryDirectory);
		} catch (IOException e) {
			Ariane.getPresenter().printErrorMessage(e.getMessage());
		}

		UserPreferences.setPreferedFileDirectory(_fileDirectory);
		UserPreferences.setBinaryDirectory(_binaryDirectory);
		UserPreferences.setTemporaryFileExtension(_temporaryFileExtension);
	}

	/**
	 * The class <code>BindingColor</code> is an inner class
	 * that represents a pair (item, color).
	 * It provides a pane of controls designed to allow a user
	 * to choose a color for a given item.
	 */
	private final class BindingColor extends JPanel {
		private final int _numColor;
		private final String _legend;
		private final JPanel _colorButton;

		/**
		 * Creates the pair (item/color).
		 * 
		 * @param legend the legend of the item.
		 * @param numColor the index of the color for the given item.
		 */
		public BindingColor( final String legend, int numColor ) {
			_legend = legend;
			_numColor = numColor;
			_colorButton = new JPanel();
			_colorButton.setPreferredSize(new Dimension(30, 30));
			_colorButton.setBorder(BorderFactory.createEtchedBorder());
			_colorButton.setBackground(_colorArray[_numColor]);
			_colorButton.addMouseListener(new MouseAdapter() {
				@Override
				public void mouseClicked( final MouseEvent event ) {
					selectColor(event.getComponent()); }
			});
			setLayout(new BorderLayout());
			JLabel label = new JLabel(_legend + "   ");
			label.setFont(new Font("Dialog", Font.PLAIN, 12));
			add(label, BorderLayout.CENTER);
			add(_colorButton, BorderLayout.EAST);
		}

		private void selectColor( Component item ) {
			final Color col = JColorChooser.showDialog(item, _legend, _colorArray[_numColor]);
			if (col != null) {
				_colorArray[_numColor] = col;
				_colorButton.setBackground(col);
				_colorButton.repaint();
			}
		}
	}
}
