/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.explorer;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.presenter.Presenter;
import fr.greyc.pantheon.ariane.view.WidgetLocalizable;
import fr.greyc.pantheon.ariane.view.utils.FadingEffects;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.KeyStroke;
import javax.swing.border.EtchedBorder;

/**
 * An explorer manages an operators tree,
 * from where user can select items to compose
 * a graph.
 * It also provides a toolbar for custom facilities (search, documentation).
 */
public final class Explorer extends JPanel implements WidgetLocalizable { // TODO
	private final ItemTreeView _itemTree;
	private final Presenter _presenter;

	private Action _searchAction;
	private JMenuItem _searchMenuItem;
	private JButton _searchButton;
	
	private Action _documentationAction;
	private JMenuItem _documentationMenuItem;
	private JButton _documentationButton;

	private JMenu _explorerMenu;

	public Explorer( Presenter presenter, ItemTreeView tree ) {
		super(new BorderLayout());
		_presenter = presenter;
		_itemTree = tree;
		add(createToolbar(), BorderLayout.NORTH);
		JScrollPane scrollPane = new JScrollPane(tree);
		scrollPane.getVerticalScrollBar().setUnitIncrement(10);
		add(scrollPane, BorderLayout.CENTER);
	}

	private JToolBar createToolbar() {
		JToolBar toolbar = new JToolBar();
		toolbar.setFloatable(false);
		toolbar.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
		toolbar.setBorderPainted(true);
		toolbar.setOpaque(false);

		_searchAction = new AbstractAction(null, Ariane.getIcon("search.png")) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				searchItemInTreeAction();
			}
		};
		_documentationAction = new AbstractAction(null, Ariane.getIcon("index.png")) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				displayDocumentationAction();
			}
		};

		_searchButton = new JButton(_searchAction);
		_searchButton.setBorderPainted(false);
		toolbar.add(_searchButton);

		_documentationButton = new JButton(_documentationAction);
		_documentationButton.setBorderPainted(false);
		toolbar.add(_documentationButton);

		return toolbar;
	}

	public JMenu getSubMenu( ) {
		_explorerMenu = new JMenu();
		_explorerMenu.setMnemonic('R');

		_searchMenuItem = new JMenuItem(_searchAction);
		_searchMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, KeyEvent.CTRL_MASK));
		_explorerMenu.add(_searchMenuItem);

		_documentationMenuItem = new JMenuItem(_documentationAction);
		_documentationMenuItem.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_I, KeyEvent.CTRL_MASK));
		_explorerMenu.add(_documentationMenuItem);

		return _explorerMenu;
	}

	@Override
	public void setInternationalText( ) {
		_searchMenuItem.setText(Ariane.getString("Menu.explorer.search"));
		_searchButton.setToolTipText(Ariane.getString("Menu.explorer.search"));
		_documentationMenuItem.setText(Ariane.getString("Menu.explorer.index"));
		_documentationButton.setToolTipText(Ariane.getString("Menu.explorer.index"));
		_explorerMenu.setText(Ariane.getString("Menu.explorer.title"));
		_itemTree.setInternationalText();
	}

	private void searchItemInTreeAction( ) {
		FadingEffects.fadeIn(Ariane.getFrame());
		new SearchDialog(_itemTree).setVisible(true);
		FadingEffects.fadeOut(Ariane.getFrame());
	}

	private void displayDocumentationAction( ) {
		_presenter.displayLibraryDocumentation();
	}
}