/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.explorer;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.model.graph.item.OperatorItem;
import fr.greyc.pantheon.ariane.model.graph.item.etc.ItemSignatureParser;
import fr.greyc.pantheon.ariane.model.tree.MutableTreeItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.view.HTMLDocumentEditor;
import fr.greyc.pantheon.ariane.view.utils.GDialog;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.lang.reflect.MalformedParameterizedTypeException;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.tree.DefaultTreeModel;

/**
 * The class <code>OperatorEditor</code> is a dialog box
 * that allows user to edit operator properties.
 */
public final class OperatorEditor extends GDialog { // TODO
	private TreeItem _currentOperator;
	private final MutableTreeItem _currentNode;
	private final DefaultTreeModel _treeModel;
	private JTextField _textfield1;
	private JTextField _textfield2;
	private JTextField _textfield2bis;
	private JTextField _docURLField;
	private JTextField _signature;
	private JSpinner _textFieldiP;
	private JSpinner _textFieldiI;
	private JSpinner _textFieldiO;

	public OperatorEditor( JDialog parent, MutableTreeItem currentNode, DefaultTreeModel treeModel ) {
		super(parent, Ariane.getString("OperatorEditor.title"), true);
		_currentNode = currentNode;
		_treeModel = treeModel;
		_currentOperator = (TreeItem)_currentNode.getUserObject();
		addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing( WindowEvent event ) {
				cancelAction();
			}
		});
		displayContent();
		setLocationRelativeTo(parent);
	}

	private void displayContent( ) {
		JPanel textPanel = new JPanel(new GridBagLayout());
		textPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("OperatorEditor.borderTitle")));

		GridBagConstraints cst = new GridBagConstraints();
		cst.fill = GridBagConstraints.BOTH;
		cst.anchor = GridBagConstraints.WEST;
		textPanel.add(new JLabel(Ariane.getString("OperatorEditor.name")), cst);
		_textfield1 = new JTextField(30);
		_textfield1.setText(_currentOperator.getName());

		cst.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(_textfield1, cst);

		cst.gridwidth = 1;
		textPanel.add(new JLabel(Ariane.getString("OperatorEditor.binary")), cst);
		_textfield2 = new JTextField(30);
		String binary = GraphItem.getAbsoluteBinary(OperatorItem.getCommandPart(_currentOperator.getBinaryPath()));
		if (binary != null && binary.length() > 0) {
			_textfield2.setText(binary);
		}
		textPanel.add(_textfield2, cst);
		JButton button1 = createButtonLoad(_textfield2, "");
		cst.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(button1, cst);

		cst.gridwidth = 1;
		textPanel.add(new JLabel(Ariane.getString("OperatorEditor.synopsis")), cst);
		_textfield2bis = new JTextField(35);
		String format = OperatorItem.getFormatPart(_currentOperator.getBinaryPath());
		_textfield2bis.setText(format);
		cst.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(_textfield2bis, cst);

		cst.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(new JLabel("<html><i>" + Ariane.getString("OperatorEditor.synopsisDescription") + "</i><html>"), cst);

		cst.gridwidth = 1;
		textPanel.add(new JLabel(Ariane.getString("OperatorEditor.manual")), cst);
		_docURLField = new JTextField(35);
		_docURLField.setText(_currentOperator.getDocumentationURL());
		textPanel.add(_docURLField, cst);
		JButton button2 = createButtonLoad(_docURLField, "file://");

		JButton button3 = new JButton(Ariane.getString("OperatorEditor.edit"));
		button3.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				HTMLDocumentEditor editor = new HTMLDocumentEditor(OperatorEditor.this);
				if (_docURLField.getText().length() > 6) {
					editor.setSelectedFile(new File(_docURLField.getText().substring(7)));
				}
				if (editor.edit() == HTMLDocumentEditor.OK_OPTION && editor.getSelectedFile() != null ) {
					_docURLField.setText("file://" + editor.getSelectedFile().getAbsolutePath());
					_docURLField.setCaretPosition(0);
				}
			}
		});
		textPanel.add(button2, cst);
		cst.gridwidth = GridBagConstraints.REMAINDER; //end row
		textPanel.add(button3, cst);

		cst.gridwidth = 1;
		cst.fill = GridBagConstraints.NONE;

		textPanel.add(new JLabel(Ariane.getString("OperatorEditor.parameters")), cst);
		SpinnerModel spModel = new SpinnerNumberModel(_currentOperator.getNumberOfParameterSlots(), 0, 10, 1);
		_textFieldiP = new JSpinner(spModel);
		cst.gridwidth = GridBagConstraints.REMAINDER;
		textPanel.add(_textFieldiP, cst);

		cst.gridwidth = 1;
		textPanel.add(new JLabel(Ariane.getString("OperatorEditor.signature")), cst);
		_signature = new JTextField(30);
		String defaultValues = _currentOperator.getSignature();
		_signature.setText(defaultValues);
		cst.gridwidth = GridBagConstraints.REMAINDER;
		textPanel.add(_signature, cst);
		textPanel.add(new JLabel("<html><i>" + Ariane.getString("OperatorEditor.signatureExample") + "</i></html>"), cst);

		cst.gridwidth = 1;
		textPanel.add(new JLabel(Ariane.getString("OperatorEditor.inputs")), cst);
		spModel = new SpinnerNumberModel(_currentOperator.getNumberOfInputDataSlots(), 0, 10, 1);
		_textFieldiI = new JSpinner(spModel);
		cst.gridwidth = GridBagConstraints.REMAINDER;
		textPanel.add(_textFieldiI, cst);

		cst.gridwidth = 1;
		textPanel.add(new JLabel(Ariane.getString("OperatorEditor.outputs")), cst);
		spModel = new SpinnerNumberModel(_currentOperator.getNumberOfOutputDataSlots(), 0, 10, 1);
		_textFieldiO = new JSpinner(spModel);
		cst.gridwidth = GridBagConstraints.REMAINDER;
		textPanel.add(_textFieldiO, cst);

		// cancel button
		JButton cancelButton = new JButton(new AbstractAction(Ariane.getString("OperatorEditor.cancel"), Ariane.getIcon("gtk-cancel.png")) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				cancelAction();
			}
		});

		JButton closeButton = new JButton(Ariane.getString("OperatorEditor.validate"), Ariane.getIcon("gtk-apply.png"));
		JPanel buttonPanel = new JPanel();
		closeButton.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				applyAction();
			}
		});
		buttonPanel.add(closeButton);
		buttonPanel.add(cancelButton);
		setDefaultButton(closeButton);
		setCancelButton(cancelButton);

		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(textPanel, BorderLayout.CENTER);
		getContentPane().add(buttonPanel, BorderLayout.SOUTH);
		pack();
	}

	private JButton createButtonLoad( final JTextField textfield, final String prefix ) {
		JButton button = new JButton(Ariane.getString("OperatorEditor.load"));
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				int prefixLength = prefix.length();
				JFileChooser fileChooser = new JFileChooser(".");
				fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
				if (textfield.getText().length() > prefixLength) {
					fileChooser.setSelectedFile(new File(textfield.getText().substring(prefixLength + 1)));
				}
				if (fileChooser.showOpenDialog(OperatorEditor.this) == JFileChooser.APPROVE_OPTION) {
					textfield.setText(prefix + fileChooser.getSelectedFile().getAbsolutePath());
					textfield.setCaretPosition(0);
				}
			}
		});
		return button;
	}

	private void cancelAction( ) {
		setVisible(false);
		dispose();
	}

	private void applyAction( ) {
		try {
			_currentOperator.setName(_textfield1.getText());
			_currentOperator.setBinaryPath(_textfield2.getText() + OperatorItem.SEPARATOR + _textfield2bis.getText());
			_currentOperator.setDocumentationURL(_docURLField.getText());
			_currentOperator.setNumberOfParameterSlots((Integer)_textFieldiP.getValue());
			_currentOperator.setNumberOfInputDataSlots((Integer)_textFieldiI.getValue());
			_currentOperator.setNumberOfOutputDataSlots((Integer)_textFieldiO.getValue());
			_treeModel.nodeChanged(_currentNode);
			ItemSignatureParser.parse(_signature.getText(), _currentOperator.getNumberOfParameterSlots());
			_currentOperator.setSignature(_signature.getText());
		} catch (NumberFormatException e ) {
			Ariane.getPresenter().printErrorMessage(e.getLocalizedMessage());
		} catch (MalformedParameterizedTypeException e) {
			Ariane.getPresenter().printErrorMessage(Ariane.getString("OperatorEditor.error1"));
		}
		setVisible(false);
		_currentOperator = null;
		dispose();
	}
}