/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.explorer;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.tree.MutableTreeItem;
import fr.greyc.pantheon.ariane.model.tree.TreeItem;
import fr.greyc.pantheon.ariane.model.xml.XMLElement;
import fr.greyc.pantheon.ariane.model.xml.XMLFileOutputStream;
import fr.greyc.pantheon.ariane.view.utils.GDialog;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

/** 
 * The class <code>SubroutineTreeEditor</code> defines a dialog box
 * where user can modify the subroutine tree.
 */
public final class SubroutineTreeEditor extends TreeEditor { // TODO

	public SubroutineTreeEditor( MutableTreeItem rootNode ) {
		super(Ariane.getString("SubroutineTreeEditor.title"), rootNode);
	}

	@Override
	protected JDialog getItemEditor( GDialog dialog, MutableTreeItem currentNode, DefaultTreeModel treeModel ) {
		return new SubroutineEditor(dialog, currentNode, treeModel);
	}

	@Override
	protected TreeItem createItem( String categoryName ) {
		return null;
	}

	@Override
	protected boolean isNewEnabled( TreePath path ) {
		return path != null && path.getPathCount() < 2;
	}

	@Override
	protected void openPopupMenuCategory( final JTree tree, int x, int y ) {
		JPopupMenu popupMenu = new JPopupMenu();

		JMenuItem item = new JMenuItem(Ariane.getString("TreeEditor.editCategory"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				editCurrentItem();
			}
		});
		popupMenu.add(item);
		popupMenu.add(item);
		item = new JMenuItem(Ariane.getString("TreeEditor.delete"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				deleteSelectedItem();
			}
		});
		popupMenu.add(item);
		popupMenu.show(tree, x, y);
	}

	@Override
	protected void openPopupMenuItem( final JTree tree, int x, int y ) { 
		JPopupMenu popupMenu = new JPopupMenu();
		JMenuItem item = new JMenuItem(Ariane.getString("TreeEditor.modify"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				editCurrentItem();
			}
		});
		popupMenu.add(item);
		item = new JMenuItem(Ariane.getString("TreeEditor.delete"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				deleteSelectedItem();
			}
		});
		popupMenu.add(item);
		popupMenu.show(tree, x, y);
	}

	@Override
	protected boolean storeInPreferenceFile( MutableTreeItem node ) {
		Ariane.getPresenter().getTree().setSubroutineTree(node);
		return save(node);
	}

	private static XMLElement convertToXMLTree() throws IOException {
		List<TreeItem> subRoutineList = Ariane.getPresenter().getTree().getSubroutineTreeAsList();
		if (!subRoutineList.isEmpty()) {
			XMLElement element = new XMLElement("SubroutineLibrary");
			for (TreeItem item : subRoutineList) {
				element.addChildren(item.convertToXMLTree());
			}
			return element;
		}
		return null;
	}

	public static boolean save( MutableTreeItem subtree ) {
		File file = new File(Ariane.getSubroutineFilename());
		File directory = file.getParentFile();
		try {
			directory.mkdirs();
			XMLFileOutputStream out = new XMLFileOutputStream(file, null);
			out.write(convertToXMLTree());
			out.close();
		} catch (IOException e) {
			Ariane.getPresenter().printErrorMessage(Ariane.getString("PreferenceChooser.error.saved") + " " + Ariane.getPreferencesFilename() + " " + e.getLocalizedMessage());
			return false;
		}
		return true;
	}
}