/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.graph;

import java.awt.Point;

/**
 * A <code>XYConstraint</code> defines a constraint that can be
 * specified for locating a component in a XYLayout.
 * A constraints can be either:
 * <ul>
 * <li>absolute location: x,y
 * <li>orientation: NORTHWEST, NORTHEAST, ..
 * </ul>
 */
public final class XYConstraint implements Cloneable {
	private XYLayout.Orientation _orientation;
	private final int _rank;
	private Point _location;

	public XYConstraint( ) {
		_rank = -1;
		_location = new Point(0, 0);
	}

	public XYConstraint( Point coordinates ) {
		_rank = -1;
		_location = new Point(coordinates.x, coordinates.y);
	}

	public XYConstraint( XYLayout.Orientation orientation, int rank ) {
		_orientation = orientation;
		_rank = rank;
	}

	public int getRank( ) {
		return _rank; 
	}

	public XYLayout.Orientation getOrientation( ) {
		return _orientation;
	}

	public Point getLocation( ) {
		return _location;
	}

	@Override
	public boolean equals( Object that ) {
		XYConstraint other = (XYConstraint)that;
		return other._location.equals(_location) && other._rank == _rank && other._orientation == _orientation;
	}
	
	@Override
	public int hashCode() {
		int code = 17;
		code = code * 37 + _orientation.ordinal();
		code = code * 37 + _rank;
		code = code * 37 + _location.x;
		code = code * 37 + _location.y;
		return code;
	}
}