/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.AbstractAction;
import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.item.ControlItem;
import fr.greyc.pantheon.ariane.model.graph.slot.InputParameterSlot;
import fr.greyc.pantheon.ariane.view.utils.FadingEffects;
import fr.greyc.pantheon.ariane.view.utils.GDialog;

public abstract class ControlItemView extends GraphItemView { // TODO
	private boolean _validatedPrototype;

	///// 
	///// P R O P E R T I E S
	/////

	public final void askProperties( String name, int defaultInputImageNumber, int defaultParameterNumber, int defaultOutputImageNumber ) throws InterruptedException {
 		final GDialog dialog = new GDialog((JFrame)null, getModel().getTagName(), true);
		dialog.setAlwaysOnTop(true);
		dialog.setResizable(false);
		_validatedPrototype = false;

		String nickName;
		if (name == null) {
			nickName = getName();
		} else {
			nickName = name;
		}
		final JTextField textFieldname = new JTextField(nickName, 10);
		SpinnerModel spModel;
		spModel = new SpinnerNumberModel(defaultParameterNumber, 0, 10, 1);
		final JSpinner textFieldiP = new JSpinner(spModel);
		spModel = new SpinnerNumberModel(defaultInputImageNumber, 0, 10, 1);
		final JSpinner textFieldiIm = new JSpinner(spModel);
		spModel = new SpinnerNumberModel(defaultOutputImageNumber, 0, 10, 1);
		final JSpinner textFieldoIm = new JSpinner(spModel);

		JPanel jPanel = new JPanel(new GridBagLayout());
		jPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), Ariane.getString("MacroOperator.borderTitle")));

		GridBagConstraints constraint = new GridBagConstraints();
		constraint.fill = GridBagConstraints.NONE;
		constraint.anchor = GridBagConstraints.WEST;
		constraint.ipadx = 8; constraint.ipady = 8;
		constraint.gridy = 0;

		if (name != null) {
			constraint.gridy++; constraint.gridx = 0;
			jPanel.add(new JLabel(Ariane.getString("MacroOperator.name")), constraint);
			constraint.gridx = 1;
			jPanel.add(textFieldname, constraint);
		}
		if (defaultParameterNumber > 0) {
			constraint.gridy++; constraint.gridx = 0;
			jPanel.add(new JLabel(Ariane.getString("MacroOperator.parameternumber")), constraint);
			constraint.gridx = 1;
			jPanel.add(textFieldiP, constraint);
		}
		if (defaultInputImageNumber > 0) {
			constraint.gridy++; constraint.gridx = 0;
			jPanel.add(new JLabel(Ariane.getString("MacroOperator.inputnumber")), constraint);
			constraint.gridx = 1;
			jPanel.add(textFieldiIm, constraint);
		}
		if (defaultOutputImageNumber > 0) {
			constraint.gridy++; constraint.gridx = 0;
			jPanel.add(new JLabel(Ariane.getString("MacroOperator.outputnumber")), constraint);
			constraint.gridx = 1;
			jPanel.add(textFieldoIm, constraint);
		}

		JButton applyButton = new JButton(Ariane.getString("PropertyViewer.ok"), Ariane.getIcon("gtk-yes.png"));

		applyButton.addActionListener(new ActionListener() {
			@Override
			public final void actionPerformed( ActionEvent event ) {
				int ii = (Integer)textFieldiIm.getValue();
				int ip = (Integer)textFieldiP.getValue();
				int io = (Integer)textFieldoIm.getValue();
				String newName = textFieldname.getText();
				((ControlItem)getModel()).setPrototype(newName, ii, ip, io);
				_validatedPrototype = true;
				dialog.setVisible(false);
			}
		});
	
		JButton cancelButton = new JButton();
		cancelButton.setAction(new AbstractAction(Ariane.getString("SubroutineEditor.cancel"),
				Ariane.getIcon("gtk-cancel.png")) {
			@Override
			public final void actionPerformed( ActionEvent event ) {
				_validatedPrototype = false;
				dialog.setVisible(false);
			}
		});

		dialog.getContentPane().setLayout(new BorderLayout(5, 5));
		dialog.getContentPane().add(jPanel, BorderLayout.CENTER);
		JPanel panel = new JPanel();
		panel.add(applyButton);
		panel.add(cancelButton);
		dialog.setDefaultButton(applyButton);
		dialog.setCancelButton(cancelButton);
		dialog.getContentPane().add(panel, BorderLayout.SOUTH);
		dialog.pack();
		dialog.setLocationRelativeTo(Ariane.getFrame());
		FadingEffects.fadeIn(Ariane.getFrame());
		dialog.setVisible(true);
		FadingEffects.fadeOut(Ariane.getFrame());

		if (!_validatedPrototype) {
			throw new InterruptedException();
		}
	}

	///// 
	///// P R I N T I N G
	/////

	@Override
	public final void printComponent( Graphics gc ) {
		super.printComponent(gc);
		gc.setColor(Color.black);
		gc.setFont(UserPreferences.getDefaultFont());
		FontMetrics fm = gc.getFontMetrics();
		int row = fm.getHeight();
		// Name
		super.centerText(getModel().getNickName(), gc, row, false);

		// Print each parameters.
		// Print the value only of if the parameter is not connected.
		for (int i = 0; i < getModel().getInputParameterSlots().size(); i++) {
			InputParameterSlot param = getModel().getInputParameterSlots().get(i);
			row += fm.getHeight() + 1;
			if (param.isConnected()) {
				super.centerText("$" + i + ":<linked>", gc, row, true);
			} else {
				super.centerText("$" + i + ":" + param.getValue(), gc, row, true);
			}
		}
		row += fm.getHeight() + 2;
		if (getFormula() != null) {
			super.centerText(getFormula(), gc, row, true);
		}
	}
}