/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */
package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.Color;
import java.awt.Graphics;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JTextField;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.graph.item.DataFile;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem.Status;
import fr.greyc.pantheon.ariane.model.graph.item.etc.OperatorProcessBuilder;
import fr.greyc.pantheon.ariane.view.graph.item.etc.DataFileColor;

public abstract class DataFileView extends GraphItemView { // TODO
	private final String _windowTitle;

	public DataFileView( String windowTitle) {
		_windowTitle = windowTitle;
	}

	public abstract void changeBinaryName( JTextField textfield1, String binary, boolean addInHistory );

	protected abstract void resetButtonText( JButton button );

	public final void displayImage( ) throws ArianeRuntimeException, InterruptedException, IOException {
		if (UserPreferences.getCommandVisu().isEmpty()) {
			throw new ArianeRuntimeException(Ariane.getString("Operator.error5"));
		}

		String currentFilename = ((DataFile)getModel()).getCurrentFilename();
		if (currentFilename == null) {
			currentFilename = getModel().getBinary();
		}
		if (currentFilename == null) {
			return;
		}
		File file = new File(currentFilename);
		if (getModel().getExecutionStatus() == Status.EXECUTED || file.exists()) {
			ArrayList<String> commandLine = new ArrayList<String>();
			commandLine.add(((DataFile)getModel()).getViewerCommand());
			commandLine.add(currentFilename);
			OperatorProcessBuilder processBuilder = new OperatorProcessBuilder(commandLine);
			Process process = processBuilder.start();

			// Check execution consistency
			InputStream inputStream = process.getInputStream();
			int exitValue = process.waitFor();

			byte[] output = new byte[inputStream.available()];
			inputStream.read(output);
			String outputString = new String(output);
			if (exitValue != 0) {
				// Problem: cannot get the system error message
				// such as Segmentation Fault, Arithmetic Exception
				if (outputString.isEmpty()) {
					outputString = Ariane.getString("Operator.error2");
				}
				throw new ArianeRuntimeException(outputString);
			} else if (!outputString.isEmpty()) {
				// OK
				Ariane.getPresenter().printOutputMessage(outputString);
			}
		}
	}

	///// 
	///// P R I N T I N G 
	/////

	@Override
	public final void printComponent( Graphics gc ) {
		super.printComponent(gc);

		gc.setColor(new Color(0, 0, 0));
		gc.setFont(UserPreferences.getDefaultFont());

		int row = gc.getFontMetrics().getHeight();

		super.centerText(Ariane.getString(_windowTitle), gc, row, false);
	}

	///// 
	///// C O L O R
	/////
	@Override
	public final Color getBackgroundColor( ) {
		return DataFileColor.getColor();
	}
}
