/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.KeyStroke;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.view.graph.item.etc.ControlColor;

public class DefaultControlItemView extends ControlItemView { // TODO

	@Override
	protected final JPopupMenu initPopup( ) {
		JPopupMenu popupMenu = new JPopupMenu();

		JMenuItem item;
		item = new JMenuItem(Ariane.getString("Graph.Menu.copy"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				Ariane.getPresenter().copy();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("Graph.Menu.cut"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				Ariane.getPresenter().cut();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		popupMenu.addSeparator();

		item = new JMenuItem(Ariane.getString("PopupMenu.execute"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getGraph().startExecution();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("PopupMenu.reset"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getModel().resetAll();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("PopupMenu.remove"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getModel().delete();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		popupMenu.add(item);

		return popupMenu;
	}

	@Override
	public final Color getBackgroundColor( ) {
		return ControlColor.getColor();
	}
}