/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JFileChooser;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.item.FileNameItem;
import fr.greyc.pantheon.ariane.model.graph.item.IScriptArgument;
import fr.greyc.pantheon.ariane.presenter.command.ChangeArgumentValueEdit;
import fr.greyc.pantheon.ariane.view.PropertyPanel;

public final class FileNameItemView extends DefaultControlItemView { // TODO
	private JCheckBox _checkBox;
	private JButton _browser;

	@Override
	protected void onMouseDoubleClicked( MouseEvent event ) {
		if (getFormula().isEmpty()) {
			getBrowser().doClick();
		} else {
			String command = UserPreferences.getTextEditorPath();
			if (command != null && !command.isEmpty()) {
				try {
					Runtime.getRuntime().exec(command + " " + getFormula());
				} catch (IOException e1) {
					Ariane.getPresenter().printErrorMessage(Ariane.getString("FileNameItem.error1") + " " + command);
				}
			} else {
				Ariane.getPresenter().printErrorMessage(Ariane.getString("FileNameItem.error2"));
			}
		}
	}

	private JButton getBrowser() {
		return _browser;
	}

	@Override
	public JComponent getFormulaEditor( ) {
		_browser = new JButton();
		_browser.setText(getModel().getFormulaText());
		_browser.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				final String currentFile = getFormula();
				final JFileChooser fileChooser = new JFileChooser();
				if ( currentFile == null
						|| currentFile.trim().isEmpty()
						|| !(new File(currentFile).exists())) {
					fileChooser.setCurrentDirectory(new File(UserPreferences.getCurrentDirectory()));
				} else {
					fileChooser.setSelectedFile(new File(currentFile));
				}
				fileChooser.setDialogType(JFileChooser.OPEN_DIALOG);
				fileChooser.setDialogTitle(Ariane.getString("FileChooser.titleOpenFile"));
				int returnVal = fileChooser.showOpenDialog(Ariane.getFrame());
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					getModel().changeFormula(_browser, fileChooser.getSelectedFile().getAbsolutePath(), true);
				}
			}
		});

		return _browser;
	}

	@Override
	public ArrayList<PropertyPanel.PropertyCouple> getSpecificProperties( ) {
		ArrayList<PropertyPanel.PropertyCouple> properties = new ArrayList<PropertyPanel.PropertyCouple>();
		final IScriptArgument model = (FileNameItem)getModel();

		_checkBox = new JCheckBox();
		_checkBox.setSelected(model.isArgumentOfRelatedScript());
		_checkBox.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				model.setArgumentOfRelatedScript(((JCheckBox)event.getSource()).isSelected());
				getGraph().setModified(true);
				getGraph().validateOneAction(new ChangeArgumentValueEdit(model, _checkBox, model.isArgumentOfRelatedScript()));
			}
		});
		properties.add(new PropertyPanel.PropertyCouple(Ariane.getString("PropertyViewer.argument"), _checkBox));
		return properties;
	}
}