/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.Component;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.item.DataFile;
import fr.greyc.pantheon.ariane.model.graph.item.LoaderItem;
import fr.greyc.pantheon.ariane.presenter.command.ChangeDatafileEdit;
import fr.greyc.pantheon.ariane.view.PropertyPanel;
import fr.greyc.pantheon.ariane.view.utils.FileImagePreviewer;
import fr.greyc.pantheon.ariane.view.utils.FileTypeViewer;

public final class LoaderItemView extends DataFileView {
	private final JButton _jbuttonSelectFile = new JButton();
	private final JTextField _binaryEditor = new JTextField("");
	private static StaticOpenFileChooser _fileChooser = new StaticOpenFileChooser();
	private static String _currentLoadingFolder;

	public LoaderItemView() {
		super("loader");
	}

	@Override
	protected void onMouseDoubleClicked( MouseEvent event ) {
		selectDataFile();
	}

	@Override
	protected void resetButtonText( JButton button ) {
		button.setText(Ariane.getString("PropertyViewer.load") + "...");
	}

	private void selectDataFile( ) {
		final String currentFile = getModel().getBinary();
		if (currentFile == null || currentFile.trim().isEmpty() || !(new File(currentFile).exists())) {
			if (_currentLoadingFolder == null) {
				_currentLoadingFolder = UserPreferences.getCurrentDirectory();
			}
			_fileChooser.setCurrentDirectory(new File(_currentLoadingFolder));
			_fileChooser.setSelectedFile(null);
		} else {
			File file = new File(currentFile);
			_fileChooser.setSelectedFile(file);
		}
		if (_fileChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
			String path = _fileChooser.getSelectedFile().getPath();
			((LoaderItem)getModel()).setCurrentFilename(null);
			_currentLoadingFolder = _fileChooser.getSelectedFile().getParent();
			changeBinaryName((JTextField)getNameEditor(), path, true);
		}
	}

	@Override
	public JComponent getBinaryEditor() {
		return null;
	}

	@Override
	public void changeBinaryName( JTextField textfield1, String binary, boolean addInHistory ) {
		JTextField textfield2 = _binaryEditor;
		JButton button = _jbuttonSelectFile;

		getModel().resetAll();
		final String oldBinary = getModel().getBinary();
		getModel().setBinary(binary);

		if (getName().equals(getModel().getNickName())
				|| oldBinary != null && DataFile.getFileNameWithoutExtension(oldBinary).equals(getModel().getNickName())) {
			if (binary == null) {
				getModel().setNickName(null);
			} else {
				getModel().setNickName(DataFile.getFileNameWithoutExtension(binary));
			}
		}
		textfield1.setText(getModel().getNickName());
		textfield2.setText(getModel().getBinary());
		if (button != null) {
			if (binary == null) {
				resetButtonText(button);
			} else {
				button.setText(DataFile.getFileNameWithoutExtension(binary));
			}
		}

		getModel().updateName();
		getGraph().setModified(true);
		if (addInHistory) {
			getGraph().validateOneAction(new ChangeDatafileEdit((DataFile)this.getModel(), textfield1, binary, oldBinary));
		}
	}

	public void setButtonEnabled( boolean flag ) {
		_jbuttonSelectFile.setEnabled(flag);
	}

	@Override
	public List<PropertyPanel.PropertyCouple> getSpecificProperties( ) {
		String name;
		String binary = getModel().getBinary();
		if (binary == null) {
			name = "";
		} else {
			File file = new File(binary);
			name = file.getName();
		}

		if (name.isEmpty()) {
			resetButtonText(_jbuttonSelectFile);
		} else {
			_jbuttonSelectFile.setText(name);
		}
		_jbuttonSelectFile.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				selectDataFile();
			}
		});
		ArrayList<PropertyPanel.PropertyCouple> properties = new ArrayList<PropertyPanel.PropertyCouple>();
		properties.add(new PropertyPanel.PropertyCouple(Ariane.getString("PropertyViewer.fileName"), _jbuttonSelectFile));
		return properties;
	}

	///// 
	///// M E N U 
	/////

	@Override
	protected JPopupMenu initPopup( ) {
		JPopupMenu popupMenu = new JPopupMenu();
		popupMenu.setFont(UserPreferences.getFont(8).deriveFont(Font.BOLD));

		JMenuItem item;
		item = new JMenuItem(Ariane.getString("Graph.Menu.copy"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				Ariane.getPresenter().copy();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("Graph.Menu.cut"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				Ariane.getPresenter().cut();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		popupMenu.addSeparator();

		item = new JMenuItem(Ariane.getString("PopupMenu.execute"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getGraph().startExecution();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("PopupMenu.executeDirectory"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getGraph().executeFolder((LoaderItem)getModel());
			}
		});
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("PopupMenu.reset"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getModel().resetAll();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("PopupMenu.remove"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getModel().delete();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		popupMenu.add(item);

		popupMenu.addSeparator();

		item = new JMenuItem(Ariane.getString("PopupMenu.selectFile"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				selectDataFile();
			}
		});
		popupMenu.add(item);

		return popupMenu;
	}

	/**
	 * Defined to keep the size and location of the 
	 * previous opening of the JFileChooser.
	 */
	private static class StaticOpenFileChooser extends JFileChooser {
		private JDialog _dialog;

		public StaticOpenFileChooser() {
			setFileView(new FileTypeViewer());
			setDialogTitle(Ariane.getString("FileChooser.titleOpenFile"));
			setFileSelectionMode(JFileChooser.FILES_ONLY);
			setAccessory(new FileImagePreviewer(this));
		}

		@Override
		protected JDialog createDialog( Component parent ) {
			JDialog dialog = super.createDialog(parent);
			if (_dialog != null) {
				dialog.setLocation(_dialog.getLocation());
				dialog.setSize(_dialog.getSize());
			}
			_dialog = dialog;
			return dialog;
		}
	}
}