/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.Color;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JTextArea;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.presenter.command.ChangeTextEdit;

public final class ScriptItemView extends DefaultControlItemView { // TODO
	private JTextArea _scriptText;
	private boolean _visible;

	@Override
	protected JComponent createCommentEditor() {
		// Override the class in order to refresh
		// the graph when the text area is modified.
		final JTextArea scriptText = new JTextArea() {
			@Override
			public void repaint() {
				getGraph().getView().repaint();
			}
		};
		scriptText.setFont(UserPreferences.getDefaultFont());
		scriptText.setBackground(Color.WHITE);
		scriptText.setForeground(Color.BLACK);
		scriptText.setDocument(new ScriptDocument());
		scriptText.setTabSize(2);

		// Mouse events.
		scriptText.addMouseListener(new MouseAdapter( ) {
			@Override
			public void mouseReleased( MouseEvent event ) {
				if (getModel().isSelected()) {
					scriptText.getCaret().setVisible(true);
				} else {
					onMouseReleased(event);
				}
			}
		});

		scriptText.addFocusListener(this);
		scriptText.setEditable(true);
		scriptText.getCaret().setVisible(false);
		scriptText.setVisible(true);
		_visible = true;
		_scriptText = scriptText;
		return scriptText;
	}

	@Override
	protected void focusGainedHook( ) {
		_scriptText.moveCaretPosition(_scriptText.getText().length());
	}

	@Override
	public void focusLost( FocusEvent event ) {
		if (event.getSource() == _scriptText) {
			String newFormula = _scriptText.getText();
			if (!newFormula.equals(getFormula()) && isShowing()) {
				getGraph().validateOneAction(new ChangeTextEdit(getModel(), _scriptText, _scriptText.getText(), getFormula()));
				setFormula(_scriptText.getText());
				getGraph().setModified(true);
			}
		}
		_scriptText.getCaret().setVisible(false);
		_scriptText.setCaretPosition(0);
	}

	@Override
	protected void onMouseDoubleClicked( MouseEvent event ) {
		_visible = !_visible;
		_scriptText.setVisible(_visible);
	}

	static class ScriptDocument extends PlainDocument {
		@Override
		public final void insertString( int offs, String str, AttributeSet attributeSet ) throws BadLocationException {
			final StringBuffer read = new StringBuffer(str);
			final StringBuffer filtered = new StringBuffer(getText(0, offs));
			String inserted = "";

			for (int i = 0; i < read.length(); i++) {
				if (read.charAt(i) == '\n' && read.length() == 1) {
					String indentation = "";
					for (int k = filtered.length() - 1; k >= 0; k--) {
						if (filtered.charAt(k) == '\n') {
							for (int l = k + 1; l < filtered.length(); l++) {
								if (filtered.charAt(l) == ' ' || filtered.charAt(l) == '\t') {
									indentation += filtered.charAt(l);
								} else {
									break;
								}
							}
							break;
						}
					}
					String toBeInserted = read.charAt(i) + indentation;
					inserted += toBeInserted;
					filtered.append(toBeInserted);
				} else {
					inserted += read.charAt(i);
					filtered.append(read.charAt(i));
				}
			}
			super.insertString(offs, inserted, attributeSet);
		}
	}
}