/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.graph.item;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTextField;
import javax.swing.KeyStroke;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.exception.ArianeRuntimeException;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem.Status;
import fr.greyc.pantheon.ariane.model.graph.item.ViewerItem;
import fr.greyc.pantheon.ariane.presenter.command.ChangeViewerCommandEdit;
import fr.greyc.pantheon.ariane.view.PropertyPanel;

public final class ViewerItemView extends DataFileView {
	private final JTextField _commandTextfield = new JTextField();

	public ViewerItemView() {
		super("viewer");
	}

	@Override
	protected void resetButtonText( JButton button ) {
		// Do nothing
	}

	@Override
	protected void onMouseDoubleClicked( MouseEvent event ) {
		if (getModel().getExecutionStatus() == Status.EXECUTED) {
			try {
				displayImage();
			} catch (ArianeRuntimeException | InterruptedException | IOException e) {
				Ariane.getPresenter().printErrorMessage(Ariane.ErrorCode.RUNTIME, getModel().getAbsoluteName(), getModel().getId(), e.getLocalizedMessage());
			}
		}
	}

	@Override
	public void changeBinaryName( JTextField textfield1, String binary, boolean addInHistory ) {
		// Empty.
	}

	public void changeCommandName( String newCommand, boolean addInHistory ) {
		final String oldCommand = getModel().getBinary();
		if (newCommand != null && newCommand.length() > 0 && !newCommand.equals(UserPreferences.getCommandVisu())) {
			getModel().setBinary(newCommand);
		} else {
			getModel().setBinary(null);
		}
		if (getModel().getBinary() == null) {
			_commandTextfield.setText(UserPreferences.getCommandVisu());
		} else {
			_commandTextfield.setText(getModel().getBinary());
		}
		getModel().getView().updatePropertyViewer();
		getGraph().setModified(true);
		getModel().resetAll();
		if (addInHistory) {
			getGraph().validateOneAction(new ChangeViewerCommandEdit((ViewerItem)getModel(), newCommand, oldCommand));
		}
	}

	///// 
	///// M E N U 
	/////

	@Override
	protected JPopupMenu initPopup( ) {
		JPopupMenu popupMenu = new JPopupMenu();
		JMenuItem item;

		item = new JMenuItem(Ariane.getString("Graph.Menu.copy"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				Ariane.getPresenter().copy();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("Graph.Menu.cut"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				Ariane.getPresenter().cut();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		popupMenu.addSeparator();

		item = new JMenuItem(Ariane.getString("PopupMenu.execute"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getGraph().startExecution();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("PopupMenu.reset"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getModel().resetAll();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_U, KeyEvent.CTRL_MASK));
		popupMenu.add(item);

		item = new JMenuItem(Ariane.getString("PopupMenu.remove"));
		item.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed( ActionEvent event ) {
				getModel().delete();
			}
		});
		item.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0));
		popupMenu.add(item);
		return popupMenu;
	}

	@Override
	public List<PropertyPanel.PropertyCouple> getSpecificProperties( ) {
		if (getModel().getBinary() == null) {
			_commandTextfield.setText(UserPreferences.getCommandVisu());
		} else {
			_commandTextfield.setText(getModel().getBinary());
		}
		_commandTextfield.addFocusListener(new FocusListener() {
			@Override
			public void focusLost( FocusEvent event ) {
				changeCommandName(((JTextField)(event.getSource())).getText().trim(), true);
			}
			@Override
			public void focusGained( FocusEvent event ) {
				// Unused
			}
		});
		ArrayList<PropertyPanel.PropertyCouple> properties = new ArrayList<PropertyPanel.PropertyCouple>();
		properties.add(new PropertyPanel.PropertyCouple(Ariane.getString("PropertyViewer.commandName"), _commandTextfield));
		return properties;
	}
}