/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 *
 *
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.graph.slot;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;
import fr.greyc.pantheon.ariane.model.graph.item.GraphItem;
import fr.greyc.pantheon.ariane.model.graph.item.etc.OperatorProcessBuilder;
import fr.greyc.pantheon.ariane.model.graph.slot.OutputDataSlot;
import fr.greyc.pantheon.ariane.view.graph.GraphView;
import fr.greyc.pantheon.ariane.view.utils.ButtonImage;
import fr.greyc.pantheon.ariane.view.utils.FileImagePreviewer;

import java.awt.Color;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComponent;

public final class OutputDataSlotView extends SlotView {
	private static final transient ImageIcon CONNECTED_ICON;
	private static final transient ImageIcon DISCONNECTED_ICON;

	static {
		CONNECTED_ICON = Ariane.getIcon("DataSlotOn.png");
		DISCONNECTED_ICON = Ariane.getIcon("DataSlotOff.png");
	}

	public OutputDataSlotView( ) {
		setIcon(DISCONNECTED_ICON);
	}

	@Override
	public void setConnectedIcon( ) {
		setIcon(CONNECTED_ICON);
	}

	@Override
	public void setDisconnectedIcon( ) {
		setIcon(DISCONNECTED_ICON);
	}

	@Override
	protected Color getLinkColor( ) {
		return GraphView.getImageLinkColor();
	}

	@Override
	public Point getAnchor( ) {
		return new Point((int)(getLocation().getX() + getWidth() + getModel().getGraphItem().getLocation().getX() + 3),
				(int)( getLocation().getY() + getHeight() / 2 + getModel().getGraphItem().getLocation().getY()));
	}

	/**
	 * Displays the result by using the visualization software
	 * given in the preferences.
	 */
	public void showDataFileContents( ) {
		if (((OutputDataSlot)getModel()).isValidate()) {
			try {
				ArrayList<String> commandLine = new ArrayList<String>();
				File file = new File(((OutputDataSlot)getModel()).getValue());
				if (file.exists() && FileImagePreviewer.isText(file, false)) {
					String command = UserPreferences.getTextEditorPath();
					if (command != null && !command.isEmpty()) {
						commandLine.add(command);
					} else {
						return;
					}
				} else {
					commandLine.add(GraphItem.getAbsoluteBinary(UserPreferences.getCommandVisu()));
				}
				commandLine.add(((OutputDataSlot)getModel()).getValue());
				OperatorProcessBuilder pb = new OperatorProcessBuilder(commandLine);
				pb.start();
			} catch (IOException e) {
				Ariane.getPresenter().printErrorMessage(Ariane.getString("Slot.error5") + ((OutputDataSlot)getModel()).getValue());
			}
		}
	}

	@Override
	public JComponent getValueEditor( ) {
		JButton button = new ButtonImage((OutputDataSlot)getModel(), Ariane.getString("OutSlotImage.display"));
		button.addActionListener(new ActionListener( ) {
			@Override
			public void actionPerformed( ActionEvent event ) {
				showDataFileContents();
			}
		});
		return button;
	}
}
