/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.utils;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LinearGradientPaint;
import java.awt.RenderingHints;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.RoundRectangle2D;

import javax.swing.JPanel;

import fr.greyc.pantheon.ariane.view.graph.item.GraphItemView;

/**
 * <code>RoundedPanel</code> is a JPanel with rounder corners.
 */
public class RoundedPanel extends JPanel {
	public static final Dimension CORNER_ARC = new Dimension(GraphItemView.ITEM_HEADER_SIZE, GraphItemView.ITEM_HEADER_SIZE);

	public RoundedPanel() {
		setOpaque(false);
	}

	@Override
	protected final void paintComponent( final Graphics graphics ) {
		final int width = getWidth();
		final int height = getHeight();
		final Graphics2D graphics2D = (Graphics2D) graphics;

		graphics2D.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);

		final Area shape = new Area(new RoundRectangle2D.Float(0, 0, width, height, CORNER_ARC.width, CORNER_ARC.height));
		final Point2D startTop = new Point2D.Float(0, 0);
		final Point2D stopBottom = new Point2D.Float(0, height);
		float offset = 12.0f;
		final float[] fractionsTopToBottom = {0.0f, offset / height, 1.0f };
		final Color[] colorsTopToBottom = {
				Color.WHITE,
				getBackground(),
				getBackground().darker()
		};
		
		final LinearGradientPaint leftToRightGradient = new LinearGradientPaint(startTop, stopBottom, fractionsTopToBottom, colorsTopToBottom);
		graphics2D.setPaint(leftToRightGradient);
		graphics2D.fill(shape);
	}
}