/* -*- c-basic-offset: 3 -*-
 *
 * Copyright (c), GREYC.
 * All rights reserved
 *
 * You may use this file under the terms of the BSD license as follows:
 *
 * "Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *   * Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *   * Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in
 *     the documentation and/or other materials provided with the
 *     distribution.
 *   * Neither the name of the GREYC, nor the name of its
 *     contributors may be used to endorse or promote products
 *     derived from this software without specific prior written
 *     permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE."
 * 
 *   
 * For more information, refer to:
 * https://clouard.users.greyc.fr/Ariane
 */

package fr.greyc.pantheon.ariane.view.utils;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.plaf.basic.BasicFileChooserUI;

import fr.greyc.pantheon.ariane.Ariane;
import fr.greyc.pantheon.ariane.model.UserPreferences;

/**
 * A dialog box for selecting the output filename.
 */
public final class ScriptNameChooser {
	private ScriptNameChooser() { }

	public static String[] chooseFilename( final String graphName, final String[][] knownScriptNames ) {
		String[] chosenFile = getFileNameFromFileChooser(graphName, knownScriptNames);
		if (chosenFile.length > 0) {
			String filename = chosenFile[0];
			File file = new File(filename);
			if (file.exists() && JOptionPane.showConfirmDialog(Ariane.getFrame(),
					Ariane.getString("Generator.dialog.message.part1")
					+ filename
					+ Ariane.getString("Generator.dialog.message.part2"),
					Ariane.getString("Generator.dialog.title"),
					JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION) {
				return new String[0];
			}
			return chosenFile;
		}
		return new String[0];
	}

	private static String[] getFileNameFromFileChooser( String filebasename, String[][] scriptNames ) {
		File currentDirName = new File(UserPreferences.getCurrentDirectory());
		final JFileChooser fileChooser = new JFileChooser();
		fileChooser.setDialogType(JFileChooser.SAVE_DIALOG);
		fileChooser.setCurrentDirectory(currentDirName);
		fileChooser.setDialogTitle(Ariane.getString("Generator.dialogchooser.title"));
		fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
		fileChooser.addPropertyChangeListener(new PropertyChangeListener() {
			@Override
			public void propertyChange( final PropertyChangeEvent event ) {
				if (JFileChooser.FILE_FILTER_CHANGED_PROPERTY.equals(event.getPropertyName())) {
					updateScriptNameFromExtension(fileChooser, event);
				}
			}
		});

		// Use file type preference
		FileNameExtensionFilter defaultFilter = null;
		final int typePreference = UserPreferences.getCodeGeneratorIndex();
		fileChooser.setAcceptAllFileFilterUsed(false);
		for (int i = 0; i < scriptNames.length; i++) {
			FileNameExtensionFilter filter = new FileNameExtensionFilter(scriptNames[i][1], scriptNames[i][2]);
			fileChooser.addChoosableFileFilter(filter);
			if (i == typePreference ) {
				defaultFilter = filter;
			}
		}
		fileChooser.setFileFilter(defaultFilter);
		// Set the first name using default extension
		String extension = defaultFilter.getExtensions()[0];
		File currentFileName = new File(filebasename + "." + extension);
		fileChooser.setSelectedFile(currentFileName);

		if (fileChooser.showSaveDialog(Ariane.getFrame()) == JFileChooser.APPROVE_OPTION) {
			// Update file type preference.
			for (int i = 0; i < scriptNames.length; i++ ) {
				if (scriptNames[i][1].equals(fileChooser.getFileFilter().getDescription())) {
					UserPreferences.getCodeGeneratorIndex(i);
				}
			}

			File theFile = fileChooser.getSelectedFile();
			if (theFile != null) {
				String fullFileName = fileChooser.getSelectedFile().getPath();
				UserPreferences.setCurrentDirectory(fileChooser.getSelectedFile().getParent());
				String description = ((FileNameExtensionFilter)fileChooser.getFileFilter()).getDescription();
				return new String[]{ fullFileName, description};
			}
		}
		return new String[0];
	}

	private static void updateScriptNameFromExtension( final JFileChooser fileChooser, PropertyChangeEvent event ) {
		JFileChooser chooser = (JFileChooser)event.getSource();
		FileNameExtensionFilter filename = (FileNameExtensionFilter)chooser.getFileFilter();
		if (filename != null) {
			String currentName = ((BasicFileChooserUI)fileChooser.getUI()).getFileName();
			if (currentName.lastIndexOf('.') > 0) {
				currentName = currentName.substring(0, currentName .lastIndexOf('.'));
			}
			final String extension = ((FileNameExtensionFilter)fileChooser.getFileFilter()).getExtensions()[0];
			final String fullFileName = currentName + "." + extension;
			fileChooser.setSelectedFile(new File(fullFileName));
		}
	}
}
